MODULE fileutil

!S get_unit ( iunit )                            returns a free FORTRAN unit number.
!F function file_exist(namefile)                 Controlla se esiste un file dal nome
!S get_extension(filename,ext,ier)               estrai l'estensione dal nome del file
!F file_rem_suffix(filename)                     rimuove l'estensione dal nome di un file
!F is_comment_line(line,s1,s2,s3)                Controlla se la linea ha un  commento all'inizio
!F s_delete_comment(line,scomm) result(str)      Elimina da line la parte commentata con scomm
!S find_key_file(fhandle,keystr,nline,line,ier)  Read file and stop when find keystr string

implicit none

   integer, parameter :: IO_MSG_LEN = 200

   type file_handle 
      private 
      integer                   :: handle_value
      integer                   :: ier = 0
      character(len=IO_MSG_LEN) :: io_msg

   contains
      procedure :: fclose
      procedure :: fopen
      procedure :: good
      procedure :: handle => get_handle
      procedure :: err
      procedure :: err_msg 

   end type file_handle

CONTAINS

   subroutine fopen(fileh,filename,ios)
!
!  Open file
!
   class(file_handle), intent(inout)      :: fileh
   character(len=*), intent(in)           :: filename
   character(len=*), intent(in), optional :: ios
   character(len=5)                       :: saction
   character(len=7)                       :: stat
   character(len=11)                      :: sform
   call get_unit(fileh%handle_value)
   saction = 'read'
   stat = 'old'
   sform = 'formatted'
   if (present(ios)) then
       if (index(ios,'w') > 0) then
           saction = 'write'
           stat = 'unknown'
       endif
       if (index(ios,'b') > 0) then
           sform = 'unformatted'
       endif
   endif
   fileh%io_msg = ' '
   open(unit=fileh%handle_value,file=filename,iostat=fileh%ier,status=stat,action=saction,form=sform,iomsg=fileh%io_msg)
   end subroutine fopen

!-------------------------------------------------------------------------------------------------------

   subroutine fclose(fileh)   
!
!  Close file
!
   class(file_handle), intent(inout) :: fileh
   close(fileh%handle_value,iostat=fileh%ier)
   end subroutine fclose

!-------------------------------------------------------------------------------------------------------
  
   logical function good(fileh)
   class(file_handle), intent(in) :: fileh
   good = fileh%ier == 0
   end function good

!-------------------------------------------------------------------------------------------------------

   integer function get_handle(fileh) result(handle)
   class(file_handle), intent(in) :: fileh
   handle = fileh%handle_value
   end function get_handle

!-------------------------------------------------------------------------------------------------------

   integer function err(fileh)
   class(file_handle), intent(in) :: fileh
   err = fileh%ier
   end function err

!-------------------------------------------------------------------------------------------------------

   function err_msg(fileh) 
   class(file_handle), intent(in) :: fileh
   character(len=IO_MSG_LEN) :: err_msg
   err_msg = fileh%io_msg
   end function err_msg

!-------------------------------------------------------------------------------------------------------

subroutine get_unit ( iunit )
!
! GET_UNIT returns a free FORTRAN unit number.
!
!  Discussion:
!
!    A "free" FORTRAN unit number is an integer between 1 and 99 which
!    is not currently associated with an I/O device.  A free FORTRAN unit
!    number is needed in order to open a file with the OPEN command.
!
!    If IUNIT = 0, then no free FORTRAN unit could be found, although
!    all 99 units were checked (except for units 5, 6 and 9, which
!    are commonly reserved for console I/O).
!
!    Otherwise, IUNIT is an integer between 1 and 99, representing a
!    free FORTRAN unit.  Note that GET_UNIT assumes that units 5 and 6
!    are special, and will never return those values.
!
!  Licensing:
!
!    This code is distributed under the GNU LGPL license. 
!
!  Modified:
!
!    18 September 2005
!
!  Author:
!
!    John Burkardt
!
!  Parameters:
!
!    Output, integer IUNIT, the free unit number.
!
  implicit none

  integer i
  integer ios
  integer iunit
  logical lopen
 
  iunit = 0
 
  do i = 1, 99
 
    if ( i /= 5 .and. i /= 6 .and. i /= 9 ) then

      inquire ( unit = i, opened = lopen, iostat = ios )
 
      if ( ios == 0 ) then
        if ( .not. lopen ) then
          iunit = i
          return
        end if
      end if

    end if
 
  end do

  return
  end subroutine get_unit

  !---------------------------------------------------------------------------------------------

   logical function file_exist(filename)
!   
!  Controlla se esiste un file dal nome
!
   character(len=*), intent(in) :: filename
!
   inquire(file=filename,exist=file_exist)
!   
   end function file_exist

  !---------------------------------------------------------------------------------------------

   subroutine file_delete(filename)
!
!  Delete file
!
   character(len=*), intent(in) :: filename
   integer                      :: stat
   integer                      :: handle
   open(newunit=handle, iostat=stat, file=filename, status='old')
   if (stat == 0) close(handle, status='delete')
   end subroutine file_delete

  !---------------------------------------------------------------------------------------------

   function get_extension(filename)  result(ext)
!
!  Estrai l'estensione dal nome del file
!
   USE STRUTIL, only:upper
   character(len=*), intent(in)      :: filename
!corr   character(len=len_trim(filename)) :: ext
   character(len=:), allocatable :: ext
   integer                           :: pos
   integer                           :: len_file,len_ext
!
   pos = index(filename,'.',back=.true.)
   len_file = len_trim(filename)
   len_ext = len_file - pos
!
   if (len_ext == 0 .or. pos == 0) then !  Se non esiste un'estensione 
       ext = ' '
   else
       ext = filename(pos+1:len_file)
   endif
!
   end function get_extension

 !----------------------------------------------------------------------------------

   character(len=1) function get_separ()
#if _WIN32
   get_separ = '\'     ! Windows
#else
   get_separ = '/'     ! Linux, Mac
#endif
   end function get_separ

 !----------------------------------------------------------------------------------

   function file_get_name(pfilename)  result(filename)
!
!  Input  : pfilename  = percorso + nome del file 
!  Output : filename   = nome del file
!
   character(len=*), intent(in)       :: pfilename
   character(len=len_trim(pfilename)) :: filename
   integer                            :: pos
!
   pos = index(pfilename,get_separ(),back=.true.)  
   if (pos > 0 .and. pos /= len(pfilename)) then
       filename = pfilename(pos+1:)
   else
       filename = ' '
   endif
!   
   end function file_get_name

  !---------------------------------------------------------------------------------------------

   function file_get_dir(pfilename) result(dir)
   character(len=*), intent(in)       :: pfilename
   character(len=len_trim(pfilename)) :: dir
   integer                            :: pos
!
   pos = index(pfilename,get_separ(),back=.true.)  
   if (pos > 0) then
       dir = pfilename(:pos)
   else
       dir = ' '
   endif
!
   end function file_get_dir

  !---------------------------------------------------------------------------------------------

   function file_rem_ext(filename)  result(sfile)
!
!  Rimuove l'estensione dal nome di un file
!   
   character(len=*), intent(in) :: filename
   character(len=len(filename)) :: sfile
   integer                      :: pos
   integer                      :: lenext
!
   sfile = filename
   pos = index(filename,'.',back=.true.)
   if (pos > 0) then                    !se esiste un'estensione
       lenext = len_trim(filename)-pos  !lunghezza dell'estensione
       if (lenext <= 3) then            !se l'estensione è <= 3 caratteri
           sfile = filename(1:pos-1)
       endif
   endif
!   
   end function file_rem_ext

  !---------------------------------------------------------------------------------------------

   logical function is_comment_line(line,s1,s2,s3)
!
!  Controlla se la linea ha un  commento del tipo s1 o s2 o s3 all'inizio
!
   character(len=*), intent(in)           :: line
   character(len=*), intent(in)           :: s1
   character(len=*), intent(in), optional :: s2,s3
   logical, dimension(3)                  :: is_comm
!
   is_comm(1) = index(line,s1) == 1
   if (present(s2)) then
       is_comm(2) = index(line,s2) == 1
   else
       is_comm(2) = .false.
   endif
   if (present(s3)) then
       is_comm(3) = index(line,s3) == 1
   else
       is_comm(3) = .false.
   endif
   is_comment_line = any(is_comm .eqv. .true.)
!
   end function

  !---------------------------------------------------------------------------------------------
  
   function s_delete_comment(line,scomm) result(str)
!
!  Elimina da line la parte commentata con scomm
!
   character(len=*), intent(in) :: line
   character(len=*), intent(in) :: scomm
   integer                      :: pos
   character(len=len(line))     :: str
!
   str = line
   pos = index(str,scomm) 
   if (pos > 0) then
       str = line(1:pos-1)
       write(0,*)'line=',trim(str)
   endif
!
   end function s_delete_comment

  !---------------------------------------------------------------------------------------------

   subroutine handle_open_error(fname,errcode,errmsg)
#if defined(__GFORTRAN__)
#elif defined(__INTEL_COMPILER)
   include 'for_iosdef.for'
#endif
   integer, intent(in)           :: errcode
   character(len=*), intent(in)  :: fname
   character(len=*), intent(out) :: errmsg
!
#if defined(__GFORTRAN__)
   errmsg = 'Error on opening file '//trim(fname)
#elif defined(__INTEL_COMPILER)
   select case (errcode)
       case (FOR$IOS_FILNOTFOU)
          errmsg = 'Cannot find file '//trim(fname)

       case (FOR$IOS_FILNAMSPE)
          errmsg = 'File '//trim(fname)//' was bad'

       case default
          errmsg = 'Error on opening file '//trim(fname)
   end select
#else
   errmsg = 'Error on opening file '//trim(fname)
#endif
!
   end subroutine handle_open_error

  !---------------------------------------------------------------------------------------------

   subroutine set_file_error(fname,errcode,error)
   USE errormod
   integer, intent(in)             :: errcode
   character(len=*), intent(in)    :: fname
   type(error_type), intent(inout) :: error
!
   call error%set()
   call handle_open_error(fname,errcode,error%message)
!
   end subroutine set_file_error

  !---------------------------------------------------------------------------------------------

   subroutine find_key_file(fhandle,keystr,nline,line,ier)
!
!  Read file and stop when find keystr string or the upto string
!
   integer, intent(in)             :: fhandle
   character(len=*), intent(in)    :: keystr
   integer, intent(inout)          :: nline
   character(len=*), intent(inout) :: line
   integer, intent(out)            :: ier
!
   do
      nline = nline + 1
      read(fhandle,'(a)',iostat=ier)line
      if (ier < 0) exit
      if(index(line,keystr) > 0 ) exit
   enddo
!
   end subroutine find_key_file

  !---------------------------------------------------------------------------------------------

   integer function run_system(command) result(ier)
#ifdef __INTEL_COMPILER
   USE IFPORT
#endif
   character(len=*), intent(in) :: command
#ifdef __INTEL_COMPILER
   ier = system(command)
#elif GCC_VERSION>=6 
   call execute_command_line(command,cmdstat=ier)
#else
   ier = 0
   call system(command)
#endif
   end function run_system

  !---------------------------------------------------------------------------------------------

   function get_recordtype(filename)  result(stype)
!
!  Determine the type of record
!
   character(len=*), intent(in) :: filename
   character(len=11)            :: stype
   type(file_handle)            :: fileh  
   character(len=500)           :: line
   character(len=*), parameter :: LF = char(10), CR = char(13)
   stype = ' '
   call fileh%fopen(filename,'b')
   if (fileh%good()) then
       read(fileh%handle()) line
       if (index(line,CR//LF) == 0 .and. index(line,CR) > 0) then
           stype = 'STREAM_CR'
       elseif (index(line,CR//LF) > 0) then
           stype = 'STREAM_CRLF'
       elseif (index(line,LF) > 0) then
           stype = 'STREAM_LF'
       endif
       call fileh%fclose()
   endif
   end function

END MODULE fileutil
