MODULE timer

   implicit none

   type timer_type 
     private
     real(kind=8)      :: sec_cpu = 0
     real(kind=8)      :: sec_elapsed = 0
     real(kind=8)      :: tot_cpu = 0
     real(kind=8)      :: tot_elapsed = 0
     logical           :: elapsed = .false.
     character(len=20) :: tname = ' '
   contains
     procedure :: reset
     procedure :: set_name
     procedure :: set_elapsed
     procedure :: tic 
     procedure :: toc
     procedure :: tprint
     procedure :: print_el
     procedure :: print_cpu
   end type

!  Usage:
!
!  type(timer_type) :: t
!  
!  call t%reset()                    ! optional
!  call t%set_name('Big procedure')  ! optional
!  call t%set_elapsed()              ! optional
!  call t%tic()
!  ...... Run big procedure .....
!  call t%toc()
!  call t%tprint(0)
!

CONTAINS

   subroutine reset(this,tname)
   class(timer_type), intent(inout)       :: this
   character(len=*), intent(in), optional :: tname
   this%sec_cpu = 0
   this%sec_elapsed = 0
   this%tot_cpu = 0
   this%tot_elapsed = 0
   if (present(tname)) call this%set_name(tname)
   end subroutine reset

!-------------------------------------------------------------------------

   subroutine set_name(this,tname)
   class(timer_type), intent(inout) :: this
   character(len=*), intent(in)     :: tname
   this%tname = tname
   end subroutine set_name

!-------------------------------------------------------------------------

   subroutine set_elapsed(this,el)
   class(timer_type), intent(inout) :: this
   logical, intent(in), optional    :: el
   if (present(el)) then
       this%elapsed = el
   else
       this%elapsed = .true.
   endif
   end subroutine set_elapsed

!-------------------------------------------------------------------------

   function wtime ( )
!  Returns a reading of the wall clock time.
!  To get the elapsed wall clock time, call WTIME before and after a given
!  operation, and subtract the first reading from the second.
   integer ( kind = 4 ) clock_max
   integer ( kind = 4 ) clock_rate
   integer ( kind = 4 ) clock_reading
   real ( kind = 8 ) wtime

   call system_clock ( clock_reading, clock_rate, clock_max )

   wtime = real ( clock_reading, kind = 8 ) &
         / real ( clock_rate, kind = 8 )

   end function wtime

!-------------------------------------------------------------------------

   subroutine tic(this)
   class(timer_type), intent(inout) :: this
   call cpu_time(this%sec_cpu)   
   if (this%elapsed) this%sec_elapsed = wtime()
   end subroutine tic

!-------------------------------------------------------------------------
   
   subroutine toc(this)
   class(timer_type), intent(inout) :: this
   real(kind=8)                     :: now
!
!  Update CPU time
   call cpu_time(now)
   this%tot_cpu = this%tot_cpu + (now - this%sec_cpu)
!
!  Update elapsed time
   if (this%elapsed) then
       this%sec_elapsed = wtime() - this%sec_elapsed
       this%tot_elapsed = this%tot_elapsed + this%sec_elapsed
   endif
!
   end subroutine toc

!-------------------------------------------------------------------------

   function secs_to_string(time) result(str)
!
!  Convert seconds in string
!
   USE strutil
   real(kind=8), intent(in)  :: time
   integer :: hh,mm
   real    :: secs
   character(len=:), allocatable :: str
!
   hh = int(time/3600.0)
   secs = time - 3600.0*hh
   mm = int(secs / 60.0)
   str = trim(i_to_s(hh))//'h '//trim(i_to_s(mm))//'m '//trim(r_to_s(secs - mm*60.0,3))//'s '
!
   end function secs_to_string

!-------------------------------------------------------------------------

   subroutine tprint(this,kpr)
   class(timer_type), intent(inout) :: this
   integer, intent(in)              :: kpr
   call this%print_cpu(kpr)
   if (this%elapsed) call this%print_el(kpr)
   end subroutine tprint

!-------------------------------------------------------------------------

   subroutine print_el(this,kpr)
   class(timer_type), intent(inout) :: this
   integer, intent(in)              :: kpr
   if (len_trim(this%tname) > 0) then
       write(kpr,'(a,a)')trim(this%tname)//' (elapsed time): '//secs_to_string(this%tot_elapsed)
   else
       write(kpr,'(a,a)')'Elapsed time: '//secs_to_string(this%tot_elapsed)
   endif
   end subroutine print_el

!-------------------------------------------------------------------------

   subroutine print_cpu(this,kpr)
   class(timer_type), intent(inout) :: this
   integer, intent(in)              :: kpr
   if (len_trim(this%tname) > 0) then
       write(kpr,'(a,a)')trim(this%tname)//' (cpu time):     '//secs_to_string(this%tot_cpu)
   else
       write(kpr,'(a,a)')'CPU time:     '//secs_to_string(this%tot_cpu)
   endif
   end subroutine print_cpu

!-------------------------------------------------------------------------

   logical function diff_time(this,time)
   type(timer_type), intent(inout) :: this
   real, intent(in) :: time
   type(timer_type) :: t_save
!
!corr   t_save = this
   call t_save%set_elapsed()
   call t_save%tic()
   diff_time = t_save%sec_elapsed - this%sec_elapsed > time
       !write(0,*)'TIME=',t_save%sec_elapsed - this%sec_elapsed
   if (diff_time) this = t_save
!
   end function diff_time

END MODULE timer
