#ifdef HAVE_CONFIG_H
#  include <config.h>
#else
#  include <config_any.h>
#endif

#include <cstdlib>
#include <cstdio>
#include <cstring>
#include <cerrno>
#include <cmath>
#include <clocale>

#include <string>
#include <sstream>
#include <deque>
#include <exception>
#include <fstream>
#include <iomanip>


#include "ilm.h"
#include <g_ext.h>



#ifdef ENABLE_NLS
#error "do not use nls"
#endif
#undef ENABLE_NLS

/*
    File names notation
    ===================

 Case Mad/Sad - Mir/Sir - Miras/Siras

     %Structure struct_task[0,1,2]
                                  .out
                                  .log
                                  .png
      MTZ:
      struct_phase.mtz         (After Mad/Mir phasing in sirprot)
      struct_edm.mtz           (After Edm in sirprot)
      struct_flunch.mtz        (After Free Lunch in sirprot, up to obs. resolution)
      struct_flunch_extra.mtz  (After Free Lunch in sirprot, up to ext. resolution)

      FRAGMENTS:
      struct_SIR200x.fra       (After task1)
      struct_SIRPROT.fra       (After task0)
      struct_n.pdb             (After task1/task0, where n=1,2,3...)
      struct_n_lsq.fra         (After sirprot refinement)

 Case Edm

     %Structure struct_edm
                          .out
                          .log
                          .png
      MTZ:
      see previous case

 Case Remo

     %Structure struct_remo
                            .out
                            .log
                            .png
      MTZ:
      struct_1_n_sir.mtz     (After sir2, where n=1,2,3...)

      FRAGMENTS:
      struct_n.pdb           (After sir, where n=1,2,3...)

 Case AbInitio

     %Structure struct_abi
                            .out
                            .log
                            .png
      MTZ:
      struct_sir.mtz         (After sir)

*/
//! \mainpage
//! \image html ilmilione.png
//! \section intro_sec Introduction
//! Il Milione is a global phasing crystallographic software suite.
//! Its capabilites include: full crystallographic processing of powder 
//! diffraction data, ab-initio phasing of single crystal diffraction data
//! of small and medium size structures, specialized protein phasing 
//! techniques such as MAD/SAD, MIR/SIR, MIRAS/SIRAS
//! \section inst_sec Installation
//! \subsection inst_unix_sec On Various UNIX platforms
//! \li Download the distribution file ilmilione-0.0.1.tar.gz
//! \li Unpack it: \n
//!   tar xfz ilmilione-0.0.1.tar.gz
//! \li Build the package: \n
//!   cd ilmilione-0.0.1 \n
//!   ./configure \n
//!   make
//! \li Install the package: \n
//!   su \n
//!   make install
//!
//! For more detailed instructions and options for the
//! ./configure script see the INSTALL file within the ilmilione-0.0.1
//! folder.
//! \subsection inst_win_sec On MS Windows
//! \li Download the Windows distribution file IlMilioneSetup.exe
//! \li Execute it.


//! The namespace of Il Milione
namespace Ilm {

   GtkProject *PTheProject = NULL;
   Application TheApp = Application();

  int Main(int argc, char *argv[])
  {
#ifdef _WIN32
     SetThreadLocale(MAKELCID(MAKELANGID(LANG_ENGLISH, SUBLANG_ENGLISH_US),SORT_DEFAULT));
//251111#else
//251111     gtk_disable_setlocale();
#endif
#ifdef MACOSX
     gtk_disable_setlocale();
#elif !defined(_WIN32)
     putenv(g_strdup("UBUNTU_MENUPROXY=0"));
#endif
    char *sl = setlocale( LC_ALL, "English" );
    if(sl == NULL)
    {
//       sl = setlocale( LC_ALL, "C" );
       sl = setlocale( LC_ALL, "" );
       struct lconv *lc = localeconv();
       if(strstr(lc->decimal_point, ","))
          sl = setlocale( LC_ALL, "C" );

#ifndef _WIN32
       putenv(strdup("LC_NUMERIC=en_US.UTF-8"));
#else
       _putenv(_strdup("LC_NUMERIC=en_US.UTF-8"));
#endif
     }
     bool result;

     try { result = TheApp.Initialize(argc, argv);}
        catch( const UserError &e )
        {
            gtkext::dlg_errmsg("%s", e.GetMessage().c_str());
            return -1;
        }
#ifndef _WIN32
    char *Licence = g_build_filename(TheApp.ConfDir().c_str(), ".lic", NULL);
    int res = AcceptLicence(Licence, TheApp.PkgDataDir());
    g_free(Licence);
    if(!res)
        return -1;
#endif

     if(!result)
     {
        try { TheApp.WriteSettings(); }
        catch( const UserError &e )
        {
            gtkext::dlg_errmsg("%s", e.GetMessage().c_str());
            return -1;
        }
        try {TheApp.ReadSettings();}
        catch( const UserError &e )
        {
          try { TheApp.WriteSettings(); }
          catch( const UserError &e )
          {
            gtkext::dlg_errmsg("%s", e.GetMessage().c_str());
            return -1;
          }
        }
        bool ifSetUp = false;
        if(argc > 1)
        {
           const gchar *fn = argv[1];
           if(strcmp(fn, "setup") == 0)
              ifSetUp = true;
        }
        try{get_examples(ifSetUp);}
        catch( const UserError &e )
        {
            gtkext::dlg_errmsg("%s", e.GetMessage().c_str());
            return -1;
        }
        TheApp.WriteSettings();
        if(ifSetUp)
           return 0;
     }
     std::string rcfile = TheApp.PkgDataDir() + "ilmdir" + G_DIR_SEPARATOR_S +"mygtkrc";
//     gtk_rc_add_default_file(rcfile.c_str());
     if(!gtk_init_check (&argc, &argv))
     {
        g_print("GUI cannot be inizialized\n");
        return -1;
     }

     PangoFontDescription *fontnew  = DefaultFont();
#ifdef USE_MY_FONT_SIZE
     pango_font_description_set_size (fontnew, MY_FONT_SIZE*PANGO_SCALE);
#endif
     GtkSettings *settings = gtk_settings_get_default();
     char *fontname = pango_font_description_to_string(fontnew);
#if GTK_MAJOR_VERSION < 3
     gtk_settings_set_string_property (settings, "gtk-font-name",
                                    fontname, "XProperty");
     gtk_rc_parse_string (g_strdup_printf("style \"user-font\" { "
                     "font_name=\"%s\" \n} \n"
                     "class \"*\" style \"user-font\"", fontname));
#else
//---------------- CSS -----------------
  GtkCssProvider *provider = gtk_css_provider_new ();
  GdkDisplay *display = gdk_display_get_default ();
  GdkScreen *screen = gdk_display_get_default_screen (display);
  gtk_style_context_add_provider_for_screen (screen, GTK_STYLE_PROVIDER (provider),     GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);

  gsize bytes_written, bytes_read;

  char *nome = g_build_filename(TheApp.PkgDataDir().c_str(), "files", "addon.css", NULL);
  GError *error = 0;
  gtk_css_provider_load_from_path (provider,
                               g_filename_to_utf8(nome, strlen(nome),
                               &bytes_read, &bytes_written, &error),
                               NULL);
  g_object_unref (provider);
  GtkIconTheme *icon_theme = gtk_icon_theme_get_default ();
  char *directory = g_build_filename(TheApp.PkgDataDir().c_str(), "pixmaps", NULL);
  gtk_icon_theme_append_search_path (icon_theme, directory);
#endif
#ifndef NOIMAGEBUTTON
      g_type_class_unref (g_type_class_ref (GTK_TYPE_BUTTON));
      g_object_set (settings, "gtk-button-images", TRUE, NULL);
#ifdef GTK_IMAGE_MENU_ITEM
      g_type_class_unref (g_type_class_ref (GTK_TYPE_IMAGE_MENU_ITEM));
      g_object_set (settings, "gtk-menu-images", TRUE, NULL);
#endif
#endif

     int w, h;
     gboolean result2 = gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &w, &h);
     if(!result2)
     {
        w = 16;
        h = 16;
     }
    TheApp.SetMenuIconSize(w, h);

    TheApp.strFontName = fontname;
    TheApp.FontDesc = fontnew;

    try {TheApp.ReadSettings();}
    catch( const UserError &e )
    {
      try { TheApp.WriteSettings(); }
      catch( const UserError &e )
      {
        gtkext::dlg_errmsg(NULL, "%s", e.GetMessage().c_str());
      }
    }
    {
      GtkWidget *SplashWnd;
      SplashWnd = create_IlmSplashWnd();
      TheApp.SetSplash(SplashWnd);
      TheApp.UpdateRecentMenu();
      TheApp.UpdateExamplesMenu();
      gtk_widget_show(SplashWnd);
      int flag = TheApp.AutoUpdate();
      if(flag)
      {
         int n;
         int res = TheApp.LastUpdate(true, &n);
         if(res > 0)
         {
              UpdateCheck(SplashWnd, res, &flag, 
                    g_strdup_printf("%d.%2.2d", n/100, n%100));
              TheApp.SetAutoUpdate(flag);
         }
         else if(res < 0)
         {
              flag = 0;
              TheApp.SetAutoUpdate(flag);
         }
      }
    }


    if(argc > 1)
    {
       const gchar *fn = argv[1];
       open_file_from_cmd_line(fn);
    }

    main_loop();
    TheApp.WriteSettings();

//  lblEnd:
    if( PTheProject!=NULL )
      delete PTheProject;
    PTheProject = NULL; // unnecessary line, being paranoid
    return 0;
  }

  
  void main_loop()
  {
  lblMainLoop: 
    try
    {
#ifndef NDEBUG
      gdk_error_trap_push();
#endif
      gtk_main ();
#ifndef NDEBUG
      gdk_flush();
      if( gdk_error_trap_pop() )
      {
        g_print("TRAP_POP X11 ERROR\n");
      }
#endif
    }
    catch( Ilm::UserError &e )
    {
      gtkext::dlg_errmsg(NULL, "%s", e.GetMessage().c_str() );
      goto lblMainLoop;
    }
    catch ( std::exception &e )
    {
      gtkext::dlg_errmsg(NULL, "%s", e.what());
      goto lblMainLoop;
    }
    catch( const char *str )
    {
      gtkext::dlg_errmsg(NULL, "%s", str);
      goto lblMainLoop;
    }
    catch( ... )
    {
      gtkext::dlg_errmsg(NULL, "%s", "unknown exception");
      goto lblMainLoop;
    }  
  }

  void edit_file(const std::string &fn)
  {
    try {
      if( !g_file_test(toUTF8(fn.c_str()), G_FILE_TEST_EXISTS) )
        throw gext::exception(fn + "  - file does not exist");
      gtkext::text_window(Ilm::PTheProject->MainWindow(), fn.c_str(), "*.sir", 
          gtkext::IFSAVE+gtkext::IFSAVEAS, fn.c_str());      
    }
    catch( const gext::exception &e )
    {
      gtkext::dlg_errmsg(NULL, e.get_message());
    }
  }

  void view_file(const std::string &fn)
  {
    try {
      if( !g_file_test(toUTF8(fn.c_str()), G_FILE_TEST_EXISTS) )
        throw gext::exception(fn + "  - file does not exist");
      gtkext::text_window(Ilm::PTheProject->MainWindow(), fn.c_str(), "*.sir", 
             gtkext::IFREADONLY, fn.c_str());      
    }
    catch( const gext::exception &e )
    {
      gtkext::dlg_errmsg(NULL, e.get_message());
    }
  }

  //! Launches internet browser
  //! \param[in] fn The name of a file to edit 
  void internet_browser(const std::string &fn)
  {
    std::string cmd(TheApp.Executables()[IExeBrowser]);
    URLOpen(fn, cmd, TheApp.SplashWidget());
  }
/*
    try {
      if( !g_file_test(toUTF8(fn.c_str()), G_FILE_TEST_EXISTS) )
        throw gext::exception(fn + "  - file does not exist");
//      std::string cmd(SZExecutables[IExeBrowser]);
      std::string cmd(TheApp.Executables()[IExeBrowser]);
      vstring args;
      args.push_back(fn);
      gext::spawn_async(cmd, args);
    }
    catch( const gext::exception &e )
    {
      gtkext::dlg_errmsg(NULL, e.get_message());
    }
  }
*/
  //! Parses file containing structure fragments
  //! \param[in] fn The filename
  void read_anom_file(const std::string &fn, std::ostream &ostr, int &ntr, const int typ)
  {
    std::ifstream f(fromUTF8(fn).c_str());
    ntr = 0;
    if( !f )
      throw UserError("failed to open file: "+fn);
    std::string line;

    while( !f.eof() )
    {
      std::getline(f, line);
//      if( g_ascii_strncasecmp(line.c_str(), "INFO", 4)==0 )
      if( g_ascii_strncasecmp(line.c_str(), "HEADER", 6)==0 )
      {
        if(typ == MAD)
           ostr << "Anomalous scatterers positions file: \n";
        else
           ostr << "Heavy atoms positions file: \n";
        ostr << line.c_str() << "\n";
      }
      if( g_ascii_strncasecmp(line.c_str(), "COORD", 5)==0 )
      {
        bool end_found = false;
        while( !f.eof() )
        {
          std::getline(f, line);
          if( g_ascii_strncasecmp(line.c_str(), "END", 3)==0 )
          {
            end_found = true;
            break;
          }
        }
        if( !end_found )
          throw UserError(fn+"\ndoes not contain substructure information");
        ++ntr;
      }
    }
    if( ntr==0 )
          throw UserError(fn+"\ndoes not contain substructure information");
    else
          ostr << "CONTENT: " << ntr << " trials" << std::endl;

  }

  void read_anom_filePDB(const std::string &fn, std::ostream &ostr, const int typ)
  {
    std::ifstream f(fromUTF8(fn).c_str());
    if( !f )
      throw UserError("failed to open file: "+fn);
    std::string line;

    while( !f.eof() )
    {
      std::getline(f, line);
      if( g_ascii_strncasecmp(line.c_str(), "HEADER", 6)==0 )
      {
        if(typ == MAD)
           ostr << "Anomalous scatterers positions file: \n";
        else
           ostr << "Heavy atoms positions file: \n";
        ostr << line.c_str() << "\n";
        return;
      }
    }
  }

  void read_PDB_file(const std::string &fn, int &nres0, int &nres1, int &numeroResidui)
  {
    std::ifstream f(fromUTF8(fn).c_str());
    std::vector<int> vresi;
    std::vector<int>::iterator it;
    bool endchain = false;
    nres0 = 1000;
    nres1 = 0;
    numeroResidui = 0;

    if( !f )
      throw UserError("failed to open file: "+fn);
    std::string line;

    while( !f.eof() )
    {
      std::getline(f, line);
      int num;
      endchain = false;
      if( g_ascii_strncasecmp(line.c_str(), "ATOM", 4)==0 )
      {
          if( IsAmino(line.substr(17, 3).c_str()) )
          {
              num = atoi(line.substr(22, 4).c_str());
              vresi.push_back(num);
          }
      }
      if( g_ascii_strncasecmp(line.c_str(), "TER", 3)==0 )
      {
          endchain = true;
          if(vresi.size() > 0)
          {
             sort(vresi.begin(), vresi.end());
             it = unique(vresi.begin(), vresi.end());
             vresi.resize( it - vresi.begin() );
             nres0 = MRmin(vresi.at(0), nres0);
             nres1 = MRmax(vresi.at(vresi.size()-1), nres1);
             numeroResidui += (int) vresi.size();
             vresi.clear();
          }
      }
    }
    f.close();
    if(!endchain)
    {
       if(vresi.size() > 0)
       {
          sort(vresi.begin(), vresi.end());
          it = unique(vresi.begin(), vresi.end());
          vresi.resize( it - vresi.begin() );
          nres0 = MRmin(vresi.at(0), nres0);
          nres1 = MRmax(vresi.at(vresi.size()-1), nres1);
          numeroResidui += (int) vresi.size();
       }
    }
  }

  /*! 
      Returns true if filenames are identical
      \param[in] fn1 The first filename
      \param[in] fn2 The second filename
  */
  bool compare_names(const std::string &fn1, const std::string &fn2)
  {
#ifdef _WIN32
    return g_ascii_strcasecmp(fn1.c_str(), fn2.c_str())==0;
#else
    return strcmp(fn1.c_str(), fn2.c_str())==0;
#endif
  }

} // namespace Ilm

namespace gext
{
  gboolean str_is_white(const gchar *str)
  {
    const gchar *ch;
    if( str==NULL )
      return TRUE;
    for( ch=str; *ch!='\0'; ++ch ) { /* budet li eto rabotat' s UTF8 ? */
      if( ((*ch)!=' ') && ((*ch)!='\t') && ((*ch)!='\v') && ((*ch)!='\r') && ((*ch)!='\n') )
        return FALSE;
    }
    return TRUE;
  }

  //! Returns full directory path of a file 
  //! \param[in] fn The filename
  std::string path_get_dirpath(const gchar *fn)
  {
    std::string fulldirname;
    if( g_path_is_absolute(fn) )
    {
      gchar *_dir = g_path_get_dirname(fn);
      fulldirname = _dir;
      g_free(_dir);
    }
    else
    {
      gchar *_curdir = g_get_current_dir(), *_full, *_dir;
      _full = g_build_path(G_DIR_SEPARATOR_S, _curdir, fn, NULL);
      g_free(_curdir);
      _dir = g_path_get_dirname(_full);
      g_free(_full);
      fulldirname = _dir;
      g_free(_dir);
    }
    return fulldirname;
  }

} // namespace gext
