#ifdef HAVE_CONFIG_H
#  include <config.h>
#else
#  include <config_any.h>
#endif

#include <cstdlib>
#include <cstdio>
#include <cstring>
#include <cerrno>
#include <cmath>

#include <string>
#include <sstream>
#include <deque>
#include <exception>
#include <fstream>
#include <cassert>
#include <iomanip>

#include "ilm.h"
#include "project.h"
#include "ilm_globals.h"
#include "util.h"
#include "application.h"
#include "g_ext.h"
#define MRmax(a,b)               ( ((a) > (b)) ? (a) : (b) )

#define MaxLen  150
namespace Ilm
{
  //! \class Project 
  //! An OS independent class containing all project data

  static const double PLAT_D[4] = {-1.00000007E-153, 1.00000007E153, -9.000001307E137, 9.000001307E-137};
  static const unsigned int PLAT_I[4] = {0xF00F1006, 0x110A0F70, 0xFF010A00, 0x001001FF};
  static const char   SIG   [ ] = { '2', '.', '0', '.', '0', '0', '\0'};

  static const char  *Res[6]    =  { "H","C","N","O","S","P"};
  static const int  RES_C[26][4] = {// C  N  O  S
                                   {   3, 1, 1, 0},     //'A'
                                   {   0, 0, 0, 0},     //'B'
                                   {   3, 1, 1, 1},     //'C'
                                   {   4, 1, 3, 0},     //'D'
                                   {   5, 1, 3, 0},     //'E'
                                   {   9, 1, 1, 0},     //'F'
                                   {   2, 1, 1, 0},     //'G'
                                   {   6, 3, 1, 0},     //'H'
                                   {   6, 1, 1, 0},     //'I'
                                   {   0, 0, 0, 0},     //'J'
                                   {   6, 2, 1, 0},     //'K'
                                   {   6, 1, 1, 0},     //'L'
                                   {   5, 1, 1, 1},     //'M'
                                   {   4, 2, 1, 0},     //'N'
                                   {   0, 0, 0, 0},     //'O'
                                   {   5, 1, 1, 0},     //'P'
                                   {   5, 2, 2, 0},     //'Q'
                                   {   6, 4, 1, 0},     //'R'
                                   {   3, 1, 2, 0},     //'S'
                                   {   4, 1, 2, 0},     //'T'
                                   {   0, 0, 0, 0},     //'U'
                                   {   5, 1, 1, 0},     //'V'
                                   {  11, 2, 2, 0},     //'W'
                                   {   0, 0, 0, 0},     //'X'
                                   {   9, 1, 2, 0},     //'Y'
                                   {   0, 0, 0, 0}      //'Z'
                                      };
  static const int  AMI_C[26][5] = {// H, C  N  O  P
                                   {  14,10, 5, 7, 1},  //'A'
                                   {   0, 0, 0, 0, 0},  //'B'
                                   {  14, 9, 3, 8, 1},  //'C'
                                   {   0, 0, 0, 0, 0},  //'D'
                                   {  28,30, 5, 9, 1},  //'E'
                                   {   0, 0, 0, 0, 0},  //'F'
                                   {  14,10, 5, 8, 1},  //'G'
                                   {   0, 0, 0, 0, 0},  //'H'
                                   {   0, 0, 0, 0, 0},  //'I'
                                   {   0, 0, 0, 0, 0},  //'J'
                                   {   0, 0, 0, 0, 0},  //'K'
                                   {   0, 0, 0, 0, 0},  //'L'
                                   {   0, 0, 0, 0, 0},  //'M'
                                   {   0, 0, 0, 0, 0},  //'N'
                                   {   0, 0, 0, 0, 0},  //'O'
                                   {  18,13, 5, 7, 1},  //'P'
                                   {   0, 0, 0, 0},     //'Q'
                                   {  22,18, 5, 7, 1},  //'R'
                                   {   0, 0, 0, 0, 0},  //'S'
                                   {  15,10, 2, 8, 1},  //'T'
                                   {  13, 9, 2, 9, 1},  //'U'
                                   {   0, 0, 0, 0, 0},  //'V'
                                   {   0, 0, 0, 0, 0},  //'W'
                                   {  27,27, 5,14, 1},  //'X'
                                   {  22,18, 5, 7, 1},  //'Y'
                                   {  13, 9, 2, 7, 1}   //'Z'
                                      };

  static int nPerResidue[6];
  //! Creates new empty project of type typ with a filename fn
  Project::Project(const std::string &fn, ProjectType typ)
  {
    gchar *dir;

    memset(this, 0, sizeof(*this));
    memcpy(szSignature, SIG, sizeof(SIG));
    memcpy(dPlatform, PLAT_D, sizeof(PLAT_D));
    memcpy(iPlatform, PLAT_I, sizeof(PLAT_I));

    eType = typ;
    dir = g_get_current_dir();
    g_strlcpy(szWorkDir, dir, sizeof(szWorkDir));
    g_free(dir);
    if( !fn.empty() )
    {
      CopiaStringa(szWorkDir, sizeof(szWorkDir), gext::path_get_dirpath(fn.c_str()).c_str());
      CopiaStringa(szStructName, sizeof(szStructName), g_path_get_basename(fn.c_str()));
      char *p = strrchr(szStructName, '.');
      if( p!=NULL ) *p = '\0';
    }
    else
    {
      dir = g_get_current_dir();
      g_strlcpy(szWorkDir, dir, sizeof(szWorkDir));
      g_free(dir);
      CopiaStringa(szStructName, sizeof(szStructName), "newproj");
    }

    nSource = 0;
    nWaveLength = 1;
    dWaveLength = 0;
    if(eType!=MOLREP && eType!=ABI )  // for MOLREP/Ab-initio it is 0
    {
      bMad1 = true;
      bMad1Auto = true;
      nMad1Refls = 0;
      
      bMad2 = true;
      iMad2SetMin = 1;
      iMad2SetMax = 10;
      bMad2Lsq = bMad2LsqPos = bMad2LsqAdp = bMad2LsqSof = true;
      dresExtra = 0.01f;
      dresFL = 0.01f;
    }
    if( eType==ABI)
      dresExtra = 0.01f;
    if( eType==MAD )
      bMad2LsqF1 = bMad2LsqF2 = true;
    if( eType==MIR || eType==MOLREP || eType==ABI)
      bNativePresent = true;
    if( eType==MOLREP)
      nMRModels = 1;
    if( eType == DEA)
      nMRModels = 2;
    SetDefaults(); // set defaults for tasks 
  }

  //! Loads existing project from file fn
  Project::Project(const std::string &fn )
  {
    std::ifstream f;
    std::string strfn;
  
    if( fn.empty() ) {
      strfn = GetFilename(PROJECT_EXT);
    }
    else
      strfn = fn;
        
    memset(this, 0, sizeof(*this));
    size_t nsz = read_project(strfn.c_str(), static_cast<char*>(static_cast<void*>(this)), sizeof(*this));

    if( nsz!=sizeof(*this) )
      throw UserError("This is not an 'ilmilione' project file: " + strfn + "\n");

    if( memcmp(szSignature, SIG, sizeof(SIG))!=0 )
      throw UserError("This project file was created by another version of 'ilmilione'\n");

    if( memcmp(dPlatform, PLAT_D, sizeof(PLAT_D))!=0 ||
      memcmp(iPlatform, PLAT_I, sizeof(PLAT_I))!=0 ) 
      throw UserError("This 'ilmilione' project file has been created on\n"
      "a computer with the different architecture\n");


    if( eType!=MAD && eType!=MIR && eType!=MRS && eType!=MOLREP && eType!=ABI && eType != EDM && eType !=DEA)
      throw UserError("Only MAD or MIR or MIRAS or MR or DEA or Ab-initio projects are supported");

    CopiaStringa(szWorkDir, sizeof(szWorkDir), gext::path_get_dirpath(strfn.c_str()).c_str());
    if( eType==MIR ) // Native is always present for MIR
      bNativePresent = true;

  }
    
  //! Loads existing project from ascii file fn
  Project::Project(bool isasc , const std::string &fn)
  {
    std::ifstream fil1;
    std::string strfn, stri;
  
    if( fn.empty() ) {
      strfn = GetFilename(PROJECT_AEXT);
    }
    else
      strfn = fn;
    int i, j, k;

    memset(this, 0, sizeof(*this));
    memcpy(szSignature, SIG, sizeof(SIG));
    memcpy(dPlatform, PLAT_D, sizeof(PLAT_D));
    memcpy(iPlatform, PLAT_I, sizeof(PLAT_I));
    fil1.open(strfn.c_str()); //, std::ios::in);
    if( !fil1 || !fil1.is_open() )
      throw UserError("Could not read file: "+strfn);
    int version;
    fil1 >> version;
#if 0
    std::string strVersion = VERSION;
    int version;
    fil1 >> version;
    if(version != MY_NUM_VERSION(strVersion))
       throw UserError("This project file was created by another version of 'ilmilione'\n");
#endif
    fil1 >> bAbiDefaults >> stri;
    fil1 >> bAbiPatterson >> stri;
    fil1 >> iCont >> stri;
    fil1 >> bEdm >> stri;
    fil1 >> bExtraDefaults >> stri;
    fil1 >> bFLDefaults >> stri;
    fil1 >> bMad1 >> stri;
    fil1 >> bMad1Auto >> stri;
    fil1 >> bMad1MorePower >> stri;
    fil1 >> bMad1Patterson >> stri;
    fil1 >> bMad1Defaults >> stri;
    for(i=0; i<MAX_WAVES; i++)
        fil1 >> bMad1Waves[i] >> stri;
    fil1 >> bMad2 >> stri;
    fil1 >> bMad2Lsq >> stri;
    fil1 >> bMad2LsqAdp >> stri;
    fil1 >> bMad2LsqF1 >> stri;
    fil1 >> bMad2LsqF2 >> stri;
    fil1 >> bMad2LsqPos >> stri;
    fil1 >> bMad2LsqSof >> stri;
    for(i=0; i<MAX_WAVES; i++)
        fil1 >> bMad2Waves[i] >> stri;
    fil1 >> bMRRes >> stri;
    fil1 >> bMRTSAuto >> stri;
    fil1 >> nMRModels >> stri;
    fil1 >> bMRUsePseudo >> stri;
    for(i=0; i<nMRModels; i++)
    {
        fil1 >> bMREdm[i] >> stri;
        fil1 >> bMRRot[i] >> stri;
        fil1 >> bMRTrans[i] >> stri;
        fil1 >> bMRUseHetero[i] >> stri;
        fil1 >> bMRPolyalanine[i] >> stri;
        fil1 >> bMRCutModel[i] >> stri;
        fil1 >> cMRAdp[i] >> stri;
    }
    fil1 >> bNativePresent >> stri;
    fil1 >> bPro >> stri;
    for(i=0; i<MAX_DERIVATIVES; i++)
        fil1 >> bProDerivs[i] >> stri;
    fil1 >> bProFlex >> stri;
    for(i=0; i<6; i++)
        fil1 >> dCell[i] >> stri;
    fil1 >> dContent >> stri;
    for(i=0; i<MAX_ELEMENTS; i++)
    {
        for(j=0; j<MAX_WAVES; j++)
        {
            for(k=0; k<2; k++)
            {
                fil1 >> dElementF1F2[i][j][k] >> stri;
            }
        }
    }
    fil1 >> dMad1Maxres >> stri;
    fil1 >> dMRTSTra >> stri;
    fil1 >> dMRTSTraLoc >> stri;
    fil1 >> dMRTSRot >> stri;
    fil1 >> dMRTSRotLoc >> stri;
    fil1 >> dMRMinres >> stri;
    fil1 >> dMRMaxres >> stri;
    for(i=0; i<nMRModels; i++)
    {
        fil1 >> dMREdmres[i] >> stri;
        fil1 >> dMRxx[i] >> stri;
        fil1 >> dMRyy[i] >> stri;
        fil1 >> dMRzz[i] >> stri;
    }
    fil1 >> dresExtra >> stri;
    fil1 >> dresFL >> stri;
    fil1 >> dresMax >> stri;
    fil1 >> dresMin >> stri;
    fil1 >> dSolventFraction >> stri;
    fil1 >> i >> stri;
    if((i < MAD) || (i > DEA))
       throw UserError("Unknown project type in file" + fn);
    eType = (ProjectType)i;
    fil1 >> iEdmFreeLunch >> stri;
    fil1 >> iEdmConv >> stri;
    for(i=0; i<4; i++)
        fil1 >> iMad2FlexDM[i] >> stri;
    fil1 >> iEdmExt >> stri;
    fil1 >> iMad2SetMax >> stri;
    fil1 >> iMad2SetMin >> stri;
    for(i=0; i<nMRModels; i++)
    {
        fil1 >> iMRalfa[i] >> stri;
        fil1 >> iMRbeta[i] >> stri;
        fil1 >> iMRgamma[i] >> stri;
        fil1 >> nMRNmol[i] >> stri;
        fil1 >> n1MR[i] >> stri;
        fil1 >> n2MR[i] >> stri;
    }
    fil1 >> iStrategy >> stri;
    fil1 >> nAbiBlocks >> stri;
    for(i=0; i<MAX_ABI_BLOCKS; i++)
    {
        fil1 >> nAbiTrialG[i] >> stri;
        fil1 >> nAbiTrialS[i] >> stri;
    }
    fil1 >> nBlocks >> stri;
    fil1 >> nDeriv >> stri;
    for(i=0; i<MAX_ELEMENTS; i++)
        fil1 >> nElementAmount[i] >> stri;
    fil1 >> nElements >> stri;
    fil1 >> nFrag >> stri;
    fil1 >> nMad1Refls >> stri;
    fil1 >> nMad1Trials1 >> stri;
    fil1 >> nMad1Trials2 >> stri;
    for(j=0; j<MAX_DERIVATIVES; j++)
    {
        for(i=0; i<MAX_ELEMENTS; i++)
        {
            fil1 >> nMirElementAmount[j][i] >> stri;
        }
        fil1 >> nMirElements[j] >> stri;
    }
    fil1 >> nMolecules >> stri;
    for(i=0; i<MAX_WAVES; i++)
        fil1 >> nMrsWaves[i] >> stri;
    fil1 >> nNrRefl >> stri;
    fil1 >> nProStore >> stri;
    fil1 >> nProTrials >> stri;
    fil1 >> nProUse >> stri;
    fil1 >> nSpgrOrder >> stri;
    fil1 >> nWaves >> stri;

    i = 1;
    while (i == 1)
    {
      fil1.getline(szJobTitle, sizeof(szJobTitle), '\n');
      i = (int) fil1.gcount();
    }
    if(gext::str_is_white(szJobTitle))
       throw UserError("Job Title not supplied in file " + fn);
    fil1.getline(szMad2File, sizeof(szMad2File), '\n');
    for(i=0; i<nMRModels; i++)
    {
        fil1.getline(szMRFile[i], sizeof(szMRFile[i]), '\n');
        if((eType == MOLREP) && gext::str_is_white(szMRFile[i]))
           throw UserError("Model PDB file not supplied in file " + fn);
    }
    fil1.getline(szMtzFile, sizeof(szMtzFile), '\n');
    if(gext::str_is_white(szMtzFile))
       throw UserError("MTZ file not supplied in file " + fn);
    fil1.getline(szSelectedColumns, sizeof(szSelectedColumns), '\n');
    fil1.getline(szSeqFile, sizeof(szSeqFile), '\n');
    fil1.getline(szSpgrName, sizeof(szSpgrName), '\n');
    if(gext::str_is_white(szSpgrName))
       throw UserError("Space group not supplied in file " + fn);
    fil1.getline(szStructName, sizeof(szStructName), '\n');
    if(gext::str_is_white(szStructName))
       throw UserError("Structure name not supplied in file " + fn);
    fil1.getline(szWorkDir, sizeof(szWorkDir), '\n');
    for(i=0; i<MAX_ELEMENTS; i++)
        fil1.getline(szElement[i], MAX_SYMBOL_LEN, '\n');
    for(j=0; j<MAX_DERIVATIVES; j++)
    {
        for(i=0; i<MAX_ELEMENTS; i++)
        {
            fil1.getline(szMirElement[j][i], MAX_SYMBOL_LEN, '\n');
        }
    }


    fil1 >> nSource >> stri;
    fil1 >> nWaveLength >> stri;
    fil1 >> dWaveLength >> stri;

    fil1 >> nBuildPgm >> stri;
    fil1 >> nCycles >> stri;
    fil1 >> bModel >> stri;
    i = 1;
    while (i == 1)
    {
      fil1.getline(szMtzFilePh, sizeof(szMtzFilePh), '\n');
      i = (int) fil1.gcount();
    }
//    fil1.getline(szMtzFilePh, sizeof(szMtzFilePh), '\n');
    if(gext::str_is_white(szMtzFilePh) && eType == DEA && !bModel)
       throw UserError("MTZ file not supplied in file " + fn);
    fil1.getline(szSelectedColumnsPh, sizeof(szSelectedColumnsPh), '\n');
    fil1.getline(szSpgrNamePh, sizeof(szSpgrNamePh), '\n');
    fil1 >> nSpgrOrderPh >> stri;
    if(gext::str_is_white(szSpgrNamePh) && eType == DEA && !bModel)
       throw UserError("Space group not supplied in file " + fn);
    for(i=0; i<6; i++)
        fil1 >> dCellPh[i] >> stri;
    fil1 >> nNrReflPh >> stri;
    fil1 >> dresMaxPh >> stri;
    fil1 >> dresMinPh >> stri;

    for(i=0; i<nMRModels; i++)
    {
        fil1 >> bMRSeq[i] >> stri;
        fil1 >> dMRvalue[i] >> stri;
    }

    fil1.close();

    CopiaStringa(szWorkDir, sizeof(szWorkDir), gext::path_get_dirpath(strfn.c_str()).c_str());
    if( eType==MIR )
      bNativePresent = true;

  }
    
  //! Sets all default values
  void Project::SetDefaults()
  {
    bMad1 = true;
    bMad1Auto = true;
    for( int i=0; i<MAX_WAVES; i++ ) bMad1Waves[i] = false,  bMad2Waves[i] = true;
    nMad1Refls = 0;
    nMad1Trials1 = nMad1Trials2 = 30;
    bMad1Defaults = true;
    nBlocks=1;
    dMad1Maxres = 0.0;
    bMad1Patterson = true; //mr false;
    bMad1MorePower = false;
    iEdmFreeLunch = 0;
    bFLDefaults = true; 
    bExtraDefaults = true;
    
    bMad2 = true;
    iMad2SetMin = 1;
    iMad2SetMax = 1;
    bMad2Lsq = bMad2LsqPos = bMad2LsqAdp = bMad2LsqSof = true;
    if( eType==MAD && nWaves>1 )
      bMad2LsqF1 = true;
    if( eType==MAD )
      bMad2LsqF2 = true;

    bEdm = true;
    iMad2FlexDM[0] = iMad2FlexDM[1] = 1; // Solvent flattening, histogram matching
    iMad2FlexDM[2] = iMad2FlexDM[3] = 0; // skeleton, etc
    iEdmConv = 0; // default convergence
    iEdmExt = 0; // default extension
    dSolventFraction = 0;

    bPro = false;
//mr 081007    bProFlex = true;
    bProFlex = false;
    for( int i=0; i<MAX_DERIVATIVES; i++ ) bProDerivs[i] = true;
    nProTrials = nProStore = nProUse = 0;
    nProTrials = -1;
    // for MOLREP
//    nMRNmol=1;
//    nMRModels = 1;
    dMRMinres=0.0;
    dMRMaxres=0.0;
    bMRRes = false;
    bMRTSAuto = true;
    bMRUsePseudo = false;                           //280409true;
    for(int i=0; i<MAX_MODELS; i++)
    {
        bMRRot[i] = 0;
        iMRalfa[i] = 0;
        iMRbeta[i] = 0;
        iMRgamma[i] = 0;
        bMRTrans[i] = 0;
        dMRxx[i] = 0.0;
        dMRyy[i] = 0.0;
        dMRzz[i] = 0.0;
        bMREdm[i] = false; //mr -1;
        bMRPolyalanine[i] = false;
        bMRCutModel[i] = false;
        n1MR[i] = 1;
        n2MR[i] = 1 ;
        cMRAdp[i] = 'W';
        bMRUseHetero[i] = true; //mr-1;
        dMREdmres[i] = 0.0;
        dMRvalue[i] = 0.0;
        bMRSeq[i] = 1;
    }
    // for Ab-initio
    bAbiPatterson=true; //mr false;
    bAbiDefaults=true;
    nAbiBlocks=3;
    for( int i=0; i<MAX_ABI_BLOCKS; i++ ) nAbiTrialG[i] = 30;
    for( int i=0; i<MAX_ABI_BLOCKS; i++ ) nAbiTrialS[i] = 30;

//new
    nFrag = 2;
    iStrategy = 1;
   
    // for DEA
    bModel = true;
    nCycles = 10;
    nBuildPgm = 0;
  }

  void Project::SaveAscii() const
  {
    FILE *fp;
    std::string strfn = GetFilename(PROJECT_AEXT);
    std::string strVersion = VERSION;
    int i, j, k;

    fp = fOpen(strfn.c_str(), "w");
    if( !fp)
      throw UserError("Could not save file: "+strfn);
    fprintf(fp, "%d \n",MY_NUM_VERSION(strVersion));
    fprintf(fp, "%d bAbiDefaults\n",bAbiDefaults?1:0 );
    fprintf(fp, "%d bAbiPatterson\n",bAbiPatterson?1:0);
    fprintf(fp, "%d iCont\n",iCont );
    fprintf(fp, "%d bEdm\n",bEdm?1:0 );
    fprintf(fp, "%d bExtraDefaults\n",bExtraDefaults?1:0 );
    fprintf(fp, "%d bFLDefaults\n",bFLDefaults?1:0 );
    fprintf(fp, "%d bMad1\n",bMad1?1:0 );
    fprintf(fp, "%d bMad1Auto\n",bMad1Auto?1:0 );
    fprintf(fp, "%d nMad1MorePower\n",bMad1MorePower?1:0 );
    fprintf(fp, "%d bMad1Patterson\n",bMad1Patterson?1:0 );
    fprintf(fp, "%d bMad1Defaults\n",bMad1Defaults?1:0 );
    for(i=0; i<MAX_WAVES; i++)
        fprintf(fp, "%d bMad1Waves(%d)\n",bMad1Waves[i]?1:0 , i+1);
    fprintf(fp, "%d bMad2\n",bMad2?1:0 );
    fprintf(fp, "%d bMad2Lsq\n",bMad2Lsq?1:0 );
    fprintf(fp, "%d bMad2LsqAdp\n",bMad2LsqAdp?1:0 );
    fprintf(fp, "%d bMad2LsqF1\n",bMad2LsqF1?1:0 );
    fprintf(fp, "%d bMad2LsqF2\n",bMad2LsqF2?1:0 );
    fprintf(fp, "%d bMad2LsqPos\n",bMad2LsqPos?1:0 );
    fprintf(fp, "%d bMad2LsqSof\n",bMad2LsqSof?1:0 );
    for(i=0; i<MAX_WAVES; i++)
        fprintf(fp, "%d bMad2Waves(%d)\n",bMad2Waves[i]?1:0 , i+1);
    fprintf(fp, "%d bMRRes\n",bMRRes?1:0 );
    fprintf(fp, "%d bMRTSAuto\n",bMRTSAuto?1:0);
    fprintf(fp, "%d nMRModels\n",nMRModels);
    fprintf(fp, "%d bMRUsePseudo\n",bMRUsePseudo?1:0 );
    for(i=0; i<nMRModels; i++)
    {
        fprintf(fp, "%d bMREdm(%d)\n",bMREdm[i]?1:0 , i+1);
        fprintf(fp, "%d bMRRot(%d)\n",bMRRot[i]?1:0 , i+1);
        fprintf(fp, "%d bMRTrans(%d)\n",bMRTrans[i]?1:0 , i+1);
        fprintf(fp, "%d bMRUseHetero(%d)\n",bMRUseHetero[i]?1:0 , i+1);
        fprintf(fp, "%d bMRPolyalanine(%d)\n",bMRPolyalanine[i]?1:0, i+1);
        fprintf(fp, "%d bMRCutModel(%d)\n",bMRCutModel[i]?1:0, i+1);
        fprintf(fp, "%c bMRAdp(%d)\n",cMRAdp[i], i+1);
    }
    fprintf(fp, "%d bNativePresent\n",bNativePresent?1:0 );
    fprintf(fp, "%d bPro\n",bPro?1:0 );
    for(i=0; i<MAX_DERIVATIVES; i++)
        fprintf(fp, "%d bProDerivs(%d)\n",bProDerivs[i]?1:0 , i);
    fprintf(fp, "%d bProFlex\n",bProFlex?1:0 );
    for(i=0; i<6; i++)
        fprintf(fp, "%lf Cell{%d}\n",dCell[i] , i+1);
    fprintf(fp, "%lf dContent\n",dContent );
    for(i=0; i<MAX_ELEMENTS; i++)
    {
        for(j=0; j<MAX_WAVES; j++)
        {
            for(k=0; k<2; k++)
            {
                fprintf(fp, "%lf dElementF1F2(%d,%d,%d)\n",dElementF1F2[i][j][k] ,
                        i+1, j+1, k+1);
            }
        }
    }
    fprintf(fp, "%lf dMad1Maxres\n",dMad1Maxres );
    fprintf(fp, "%lf dMRTSTra\n",dMRTSTra );
    fprintf(fp, "%lf dMRTSTraLoc\n",dMRTSTraLoc );
    fprintf(fp, "%lf dMRTSTSRot\n",dMRTSRot );
    fprintf(fp, "%lf dMRTSTSRotLoc\n",dMRTSRotLoc );
    fprintf(fp, "%lf dMRMinres\n",dMRMinres );
    fprintf(fp, "%lf dMRMaxres\n",dMRMaxres );
    for(i=0; i<nMRModels; i++)
    {
        fprintf(fp, "%lf dMREdmres(%d)\n",dMREdmres[i] ,i+1);
        fprintf(fp, "%lf dMRxx(%d)\n",dMRxx[i] ,i+1);
        fprintf(fp, "%lf dMRyy(%d)\n",dMRyy[i] ,i+1);
        fprintf(fp, "%lf dMRzz(%d)\n",dMRzz[i] ,i+1);
    }
    fprintf(fp, "%lf dresExtra\n",dresExtra );
    fprintf(fp, "%lf dresFL\n",dresFL );
    fprintf(fp, "%lf dresMax\n",dresMax );
    fprintf(fp, "%lf dresMin\n",dresMin );
    fprintf(fp, "%lf dSovlentFraction\n",dSolventFraction );
    fprintf(fp, "%d Type\n", eType );
    fprintf(fp, "%d iEdmFreeLunch\n",iEdmFreeLunch );
    fprintf(fp, "%d iEdmConv\n",iEdmConv );
    for(i=0; i<4; i++)
        fprintf(fp, "%d iMad2FlexDM(%d)\n",iMad2FlexDM[i] ,i+1);
    fprintf(fp, "%d iEdmExt\n",iEdmExt );
    fprintf(fp, "%d iMad2SetMax\n",iMad2SetMax );
    fprintf(fp, "%d iMad2SetMin\n",iMad2SetMin );
    for(i=0; i<nMRModels; i++)
    {
        fprintf(fp, "%d iMRalfa(%d)\n",iMRalfa[i] ,i+1);
        fprintf(fp, "%d iMRbeta(%d)\n",iMRbeta[i] ,i+1);
        fprintf(fp, "%d iMRgamma(%d)\n",iMRgamma[i] ,i+1);
        fprintf(fp, "%d iMRNmol(%d)\n", nMRNmol[i] ,i+1);
        fprintf(fp, "%d n1MR(%d)\n",n1MR[i],i+1);
        fprintf(fp, "%d n2MR(%d)\n",n2MR[i],i+1);
    }
    fprintf(fp, "%d iStrategy\n",iStrategy );
    fprintf(fp, "%d nAbiBlocks\n", nAbiBlocks );
    for(i=0; i<MAX_ABI_BLOCKS; i++)
    {
        fprintf(fp, "%d nAbiTrialG(%d)\n", nAbiTrialG[i] ,i+1);
        fprintf(fp, "%d nAbiTrialS(%d)\n", nAbiTrialS[i] ,i+1);
    }
    fprintf(fp, "%d nBlocks\n", nBlocks );
    fprintf(fp, "%d nDeriv\n", nDeriv );
    for(i=0; i<MAX_ELEMENTS; i++)
        fprintf(fp, "%d  nElementAmount(%d)\n", nElementAmount[i], i+1);
    fprintf(fp, "%d nElements\n", nElements );
    fprintf(fp, "%d nFrag\n", nFrag );
    fprintf(fp, "%d nMad1Refls\n", nMad1Refls );
    fprintf(fp, "%d nMad1Trials1\n", nMad1Trials1 );
    fprintf(fp, "%d nMad1Trials2\n", nMad1Trials2 );
    for(j=0; j<MAX_DERIVATIVES; j++)
    {
        for(i=0; i<MAX_ELEMENTS; i++)
        {
            fprintf(fp, "%d nMirElementAmount(%d,%d)\n", nMirElementAmount[j][i],j+1,i+1);
        }
        fprintf(fp, "%d nMirElements(%d)\n", nMirElements[j],j+1);
    }
    fprintf(fp, "%d nMolecules\n", nMolecules );
    for(i=0; i<MAX_WAVES; i++)
        fprintf(fp, "%d nMrsWaves(%d)\n", nMrsWaves[i],i+1);
    fprintf(fp, "%d nNrRefl\n", nNrRefl );
    fprintf(fp, "%d nProStore\n", nProStore );
    fprintf(fp, "%d nProTrials\n", nProTrials );
    fprintf(fp, "%d nProUse\n", nProUse );
    fprintf(fp, "%d nSpgrOrder\n", nSpgrOrder );
    fprintf(fp, "%d nWaves\n", nWaves );
    const char *ibl = "  ";
    if(gext::str_is_white(szJobTitle))
       fprintf(fp, "%s\n", ibl );
    else
       fprintf(fp, "%s\n", szJobTitle );
    if(gext::str_is_white(szMad2File))
       fprintf(fp, "%s\n", ibl );
    else
       fprintf(fp, "%s\n", szMad2File );
    for(i=0; i<nMRModels; i++)
    {
        if(gext::str_is_white(szMRFile[i]))
           fprintf(fp, "%s\n", ibl );
        else
           fprintf(fp, "%s\n", szMRFile[i] );
    }
    if(gext::str_is_white(szMtzFile))
       fprintf(fp, "%s\n", ibl );
    else
       fprintf(fp, "%s\n", szMtzFile );
    if(gext::str_is_white(szSelectedColumns))
       fprintf(fp, "%s\n", ibl );
    else
       fprintf(fp, "%s\n", szSelectedColumns );
    if(gext::str_is_white(szSeqFile))
       fprintf(fp, "%s\n", ibl );
    else
       fprintf(fp, "%s\n", szSeqFile );
    if(gext::str_is_white(szSpgrName))
       fprintf(fp, "%s\n", ibl );
    else
       fprintf(fp, "%s\n", szSpgrName );
    if(gext::str_is_white(szStructName))
       fprintf(fp, "%s\n", ibl );
    else
       fprintf(fp, "%s\n", szStructName );
    if(gext::str_is_white(szWorkDir))
       fprintf(fp, "%s\n", ibl );
    else
       fprintf(fp, "%s\n", szWorkDir );
    for(i=0; i<MAX_ELEMENTS; i++)
    {
       if(gext::str_is_white(szElement[i]))
          fprintf(fp, "%s\n", ibl );
       else
          fprintf(fp, "%s\n", szElement[i]);
    }
    for(j=0; j<MAX_DERIVATIVES; j++)
    {
        for(i=0; i<MAX_ELEMENTS; i++)
        {
            if(gext::str_is_white(szMirElement[j][i]))
               fprintf(fp, "%s\n", ibl );
            else
               fprintf(fp, "%s\n", szMirElement[j][i]);
        }
    }
    fprintf(fp, "%d Source\n", nSource );
    fprintf(fp, "%d nWavelength\n", nWaveLength );
    fprintf(fp, "%lf dWavelength\n", dWaveLength );

    fprintf(fp, "%d nBuildPgm\n", nBuildPgm);
    fprintf(fp, "%d nCycles\n", nCycles);
    fprintf(fp, "%d bModel\n", bModel?1:0);
    fprintf(fp, "%s\n", szMtzFilePh);
    fprintf(fp, "%s\n", szSelectedColumnsPh);
    fprintf(fp, "%s\n", szSpgrNamePh);
    fprintf(fp, "%d nSpgrOrderPh\n", nSpgrOrderPh);
    for(i=0; i<6; i++)
        fprintf(fp, "%lf dCellPh(%i)\n", dCellPh[i], i+1);
    fprintf(fp, "%d nNrReflPh\n", nNrReflPh);
    fprintf(fp, "%lf dresMaxPh\n", dresMaxPh);
    fprintf(fp, "%lf dresMinPh\n", dresMinPh);

    for(i=0; i<nMRModels; i++)
    {
        fprintf(fp, "%d bMRSeq(%d)\n",bMRSeq[i]?1:0,i+1);
        fprintf(fp, "%lf dMRvalue(%d)\n", dMRvalue[i],i+1);
    }
    fclose(fp);
  }

  //! Stores project data into a file
  void Project::Save() const
  {
    std::string strfn;
    std::ofstream fil;

    strfn = GetFilename(PROJECT_EXT);
    fil.open(strfn.c_str(), std::ios::binary|std::ios::out);
    if( !fil || !fil.is_open() )
      throw UserError("Could not save file: "+strfn);

    fil.write(static_cast<const char*>(static_cast<const void*>(this)), sizeof(*this));
    fil.close();
  }

  //! Returns number of residues
  int Project::NResidues() const
  {
    int nres;
    if( iCont > 0 )
      nres = iround( dContent );
    else
      nres = iround( dContent/DResidueWeight );
    return nres;
  }

  int Project::NResiduesMod(int i) const
  {
    return n_residui[i];
//    return max_resid[i];
  }

  //! Returns molecular weight
  double Project::MolecularWeight() const
  {
    double molves;
    if( iCont > 0 )
      molves = iround( dContent ) * DResidueWeight;
    else
      molves = dContent;
    return molves;
  }

  int Project::HeteroAtoms(std::ostream &os) const
  {
    for(int i=0; i<nElements; i++ )
      os << szElement[i] << " " << nElementAmount[i] << " ";
    return nElements;
  }

  void Project::Content(std::ostream &os) const
  {
    const int nres = NCellResidues();
    int nPerRes[6], i, j;
    if(iCont == 2)
    {
       for(i=0; i<6; i++)
           nPerRes[i] = nPerResidue[i];
    }
    else
    {
       nPerRes[0] = iround(nres*DHPerResidue);
       nPerRes[1] = iround(nres*DCPerResidue);
       nPerRes[2] = iround(nres*DNPerResidue);
       nPerRes[3] = iround(nres*DOPerResidue);
       nPerRes[5] = 0;
       if (eType != ABI)
           nPerRes[4] = iround(nres*DSPerResidue);
       else
           nPerRes[4] = 0;
    }
    for(i=0; i<6; i++)
    {
        for(j=0; j<nElements; j++ )
        {
            if(g_ascii_strcasecmp (szElement[j], Res[i]) == 0)
            {
               if((i == 4) && ((eType == MAD) || (eType == MRS)))
                   nPerRes[i] = MRmax(nPerRes[i], nElementAmount[j]*nSpgrOrder);
               else
                   nPerRes[i] += nElementAmount[j]*nSpgrOrder;
            }
        }
    }
    for(int i=0; i<6;i++)
    {
        if(nPerRes[i])
           os << "  " << Res[i] << " " << nPerRes[i];
    }
    for(j=0; j<nElements; j++ )
    {
        if(((g_ascii_strcasecmp (szElement[j], Res[0]) == 0) && nPerRes[0]) ||
           ((g_ascii_strcasecmp (szElement[j], Res[1]) == 0) && nPerRes[1]) ||
           ((g_ascii_strcasecmp (szElement[j], Res[2]) == 0) && nPerRes[2]) ||
           ((g_ascii_strcasecmp (szElement[j], Res[3]) == 0) && nPerRes[3]) ||
           ((g_ascii_strcasecmp (szElement[j], Res[4]) == 0) && nPerRes[4]) ||
           ((g_ascii_strcasecmp (szElement[j], Res[5]) == 0) && nPerRes[5]) )
              continue;
        os << " "  << szElement[j] << " " << nElementAmount[j]*nSpgrOrder << " ";
    }
  }

  int Project::HeteroAndAnom(std::ostream &os, std::ostream &os1, int *n_anom, int mad) const
  {
    int nh = 0, na = 0;
    for(int i=0; i<nElements; i++ )
    {
        bool is_anom = false;
        for( int j=0; j<nWaves; j++ )
        {
           if( std::fabs( dElementF1F2[i][j][0] ) > 0.0001 || std::fabs( dElementF1F2[i][j][1] ) > 0.0001 )
           {
               is_anom = true;
               break;
           }
        }
        if( is_anom )
        {
            os1 << "anomalous ";
            os1 << szElement[i] << " " << nElementAmount[i] << " ";
            for( int j=0; j<nWaves; j++ )
            {
//171006        if(((mad == 1) && (bMad1Auto || bMad1Waves[j])) || ((mad == 2) && bMad2Waves[j]))
                   os1 << dElementF1F2[i][j][0] << " " << dElementF1F2[i][j][1] << " ";
            }
            os1 << std::endl;
            na++;
        }
        else
        {
            os << szElement[i] << " " << nElementAmount[i] << " ";
            nh++;
        }

    }
    *n_anom = na;
    return nh;
  }

  void Project::AddFlex(std::ostream &os) const
  {
    os << "%edm\n"; 
    os << std::setprecision (2)  << "solvent " << dSolventFraction << "\n"; 
    if(iEdmFreeLunch && dresMax > dresExtramin)
          os << "FreeLunch\n";
    if (bEdm)
    {
        if(bProFlex)
        {
           os << "dm -1 1 0 0\n" ;
           os << "convc -1 \n";
           os << "refextension 0 \n";
        }
        else
        {
           os << "dm " << iMad2FlexDM[0] <<" "<< iMad2FlexDM[1] <<" "<< iMad2FlexDM[2] <<" "<< iMad2FlexDM[3] << "\n";
           os << "convc " << iEdmConv-1 << "\n";
           os << "refextension "<< iEdmExt << "\n";
        }
    }
    else
    {
        os << "dm 0 0 0 0 \n";
        os << "convc -1 \n";
        os << "refextension 0 \n";
    }
 }

  //! Returns number of non-hydrogen atoms
  int Project::NNonHProtein() const
  {
    int ntot=0;
    if(iCont == 2)                // Sequence
       for(int i=1; i<6; i++)
           ntot += nPerResidue[i];
    else
    {
       const int nres = NCellResidues();
       ntot = ( iround(nres*DCPerResidue) + iround(nres*DNPerResidue) + iround(nres*DOPerResidue) );
       if (eType != ABI)
           ntot += iround(nres*DSPerResidue);
    }
    return ntot;
  }

  int Project::NNonHtot() const
  {
    int ntot=0;
    if(iCont == 2)                // Sequence
       for(int i=1; i<6; i++)
           ntot += nPerResidue[i];
    else
    {
       const int nres = NCellResidues();
       ntot = ( iround(nres*DCPerResidue) + iround(nres*DNPerResidue) + iround(nres*DOPerResidue) );
       if (eType != ABI)
           ntot += iround(nres*DSPerResidue);
    }
    for(int i=0; i<nElements; i++ ) 
    {
      if( g_ascii_strncasecmp(szElement[i], "H", 1)!=0 )
        ntot += nElementAmount[i]*nSpgrOrder;
    }
    return ntot;
  }

  //! Returns Matthews coefficient
  double Project::Vm(int which) const
  {
    // which = 0  ----> target
    // which = 1  ----> model
    double Vmm=0.0;
    if( dContent > 0.00001 )
    {
        if(which)                 // Model
        {
          int kk = which-1;
          if(max_resid[kk])
             Vmm = CellVolume(dCell) / (DResidueWeight*NCellResiduesMod(kk));
        }
        else                     // Target
        {
          Vmm = CellVolume(dCell) / (14*NNonHProtein());
        }
    }
    return Vmm;
  }

  void Project::SaveAsSirEdm() const
  {
    std::ofstream os;
    gchar *proj_fn = NULL;

    std::string str(szStructName);
    str += TYPE_EXT[EDM-1];
    str += INP_EXT;
    proj_fn = g_build_filename(szWorkDir, str.c_str(), NULL);
    os.open(fromUTF8(proj_fn).c_str());

    if( !os || !os.is_open() )
      return;
    std::stringstream dnum;
    dnum << std::setprecision(3);
    
    os << "%window\n%structure  " << szStructName << "_edm\n%initialize\n";
    os << "%job " << szJobTitle << "\n%data\n";
    os << "mtz " << fromUTF8(szMtzFile) << "\n";
    os << "label " << szSelectedColumns << "\n";
    os << "phase\n";
    switch(iCont)
    {
    case 0:               // Molecular Weight
      os << "Weight " << dContent << "\n";
      break;
    case 1:               // Number of residues
      os << "nResidues " << iround(dContent) << "\n";
      break;
    case 2:               // Sequence
      os << "Sequence " << fromUTF8(szSeqFile) << "\n";
      break;
    }
    std::stringstream strm1;
    if(this->HeteroAtoms(strm1))
      os << "hetatm " << strm1.str().c_str() <<"\n";
    os << "nMolecules " << nMolecules << "\n";
    os << "!content "; Content(os); os << "\n";

    if( nSource == 1)
        os << "neutron \n";
    else if( nSource == 2)
        os << "electron \n";
/* 070309 
    if( nWaveLength < 2)
        os << "Wavelength " <<  Ilm::Waves[nWaveLength] << "\n";
     else if( std::fabs( dWaveLength) > 0.0001 )
        os << "Wavelength " <<  dWaveLength << "\n";
*/
    if(iMad2FlexDM[2] && dresMax > dresExtramin)
    {
      dnum << dresExtra;
      if(bExtraDefaults)
         os << "Extrapolate \n";
      else
         os << "Extrapolate " << dnum.str().c_str() << "\n";
    }
    else if(iEdmFreeLunch && dresMax > dresExtramin)
    {
      dnum << dresFL;
      if(bFLDefaults)
         os << "Extrapolate \n";
      else
         os << "Extrapolate " << dnum.str().c_str() << "\n";
    }
    AddFlex(os);
    os << "%end\n";
    os.close();
  }

  //! Writes instructions for sir, sirprot into a file fn
  int Project::SaveAsSir(int mad_m, const gchar *fn)
  {
    int i, j, mad;
    FILE *fp = NULL;
    gchar *proj_fn = NULL;
    GString *str = g_string_new(NULL);
    bool mdev = false;
    
    if(mad_m <= 3)
       mad = mad_m;
    else
    {
       mad = mad_m-3;
       mdev = true;
    }
    int task = mad % 3;

    if( !((mad==1 && bMad1) || (mad==2 && bMad2) || (mad==3 && bPro) || eType==MOLREP || eType==ABI) )
      return FALSE;

    if( fn==NULL ) 
    {
      if(eType >= EDM)
         g_string_printf(str, "%s%s%s", szStructName, TYPE_EXT[eType-1], INP_EXT);
      else
         g_string_printf(str, "%s%s%s", szStructName, TYPE_EXT[task], INP_EXT);
      proj_fn = g_build_filename(szWorkDir, str->str, NULL);
      g_string_free(str, TRUE);
      fp = fOpen(proj_fn, "w");
    }
    else
      fp = fOpen(fn, "w");

    if( fp==NULL )
      return FALSE;

    fprintf(fp, "%%window\n");
    if( (mad==1 && bMad2) || (mad==2 && bMad1) )
      fprintf(fp, "%%automatic\n");
    if(iStrategy==1 || iStrategy==2)
       fprintf(fp, "%%ilMilione %d\n", iStrategy);
    else if(iStrategy==3)
       fprintf(fp, "%%ilMilione %d\n", nFrag+3);

    if(eType < EDM)
       fprintf(fp, "%%%s %s_task%d\n", "structure", szStructName, task);
    else
       fprintf(fp, "%%%s %s%s\n", "structure", szStructName, TYPE_EXT[eType-1]);
    if(mdev && mad==1)
    {
       fprintf(fp, "%%develop\n");
       fprintf(fp, "%%end\n");
       fclose(fp);
       return TRUE;
    }
     fprintf(fp, "%%%s\n", "initialize");
     fprintf(fp, "%%%s %s\n", "job", szJobTitle);
     fprintf(fp, "%%%s\n", "data");
     fprintf(fp, "mtz %s\n", fromUTF8(szMtzFile).c_str());
     fprintf(fp, "label %s\n", szSelectedColumns);
     if(bFindSpace)
     {
        fprintf(fp, "nospace \n");
     }
     else if(g_ascii_strcasecmp(Ilm::PTheProject->MtzGetSpgrName(szMtzFile), szSpgrName) != 0)
        fprintf(fp, "space %s\n", szSpgrName);
     if(mad == 2)
     {
        if(iMad2FlexDM[2] && (dresMax > dresExtramin))
        {
            if(bExtraDefaults)
               fprintf(fp, "Extrapolate \n");
            else
               fprintf(fp, "Extrapolate %.2f\n", dresExtra);
        }
        else if(iEdmFreeLunch && (dresMax > dresExtramin))
        {
            if(bFLDefaults)
               fprintf(fp, "Extrapolate \n");
            else
               fprintf(fp, "Extrapolate %.2f\n", dresFL);
        }
     }
     if((eType == ABI) && iMad2FlexDM[2] && (dresMax > dresExtramin))
            fprintf(fp, "Extrapolate\n");
     switch(iCont)
     {
     case 0:               // Molecular Weight
         fprintf(fp, "Weight %lf\n", dContent);
       break;
     case 1:               // Number of residues
         fprintf(fp, "nResidues %d\n", iround(dContent));
       break;
     case 2:               // Sequence
         fprintf(fp, "Sequence %s\n", fromUTF8(szSeqFile).c_str());
       break;
     }

     fprintf(fp, "nMolecules %d\n", nMolecules);
     std::stringstream strm;
     this->Content(strm);
     fprintf(fp, "!%s %s\n", "content", strm.str().c_str());

     if( eType!=MAD )
     {
        std::stringstream strm1;
        if(this->HeteroAtoms(strm1))
           fprintf(fp, "hetatm %s\n", strm1.str().c_str());
     }
     if( eType==MAD )
     {
         std::stringstream strm1, strm2;
         int na;
         if((mad == 1) || (mad == 2))
         {
            if(this->HeteroAndAnom(strm1, strm2, &na, mad))
               fprintf(fp, "hetatm %s\n", strm1.str().c_str());
            if(na)
              fprintf(fp, "%s", strm2.str().c_str());
         }
     }
     else if( eType==MIR ) 
     {
         for( i=0; i<nDeriv; i++ )
         {
            if(mad == 1)
            {
               if(bMad1Auto || bMad1Waves[i])
                  fprintf(fp, "derivative %s %d 0.0 0.0\n", szMirElement[i][0], nMirElementAmount[i][0]);
            }
            else if(mad == 2)
            {
               if(bMad2Waves[i])
                  fprintf(fp, "derivative %s %d 0.0 0.0\n", szMirElement[i][0], nMirElementAmount[i][0]);
            }
            else
            {
               if(bProDerivs[i])
                  fprintf(fp, "derivative %s %d 0.0 0.0\n", szMirElement[i][0], nMirElementAmount[i][0]);
            }
         }
     }
     else if( eType==MRS ) 
     {
         for( i=0; i<nDeriv; i++ ) 
         {
              if (( (mad==1) && (bMad1Auto || bMad1Waves[i]) ) ||
                  ( (mad==2) && bMad2Waves[i]) || ( (mad==3) && bProDerivs[i])) 
              {
                  fprintf(fp, "deranom %s %d ", szMirElement[i][0], nMirElementAmount[i][0]);
                  for( j=0; j<nMrsWaves[i]; j++ )
                     fprintf(fp, " %f %f",  dElementF1F2  [i][j][0],  dElementF1F2[i][j][1]);
                  fprintf(fp, "\n");
              }
         }
     }

     if( nSource == 1)
         fprintf(fp, "%s\n", "neutron");
     else if( nSource == 2)
         fprintf(fp, "%s\n", "electron");
/* 070309
     if( nWaveLength < 2)
         fprintf(fp, "Wavelength %s\n", Ilm::Waves[nWaveLength]);
     else if( std::fabs( dWaveLength) > 0.0001 )
         fprintf(fp, "Wavelength %lf\n", dWaveLength);
*/
     if( eType==MAD ) 
     {
         if( !this->bNativePresent && mad==2 )
             fprintf(fp, "%s\n", "nofo");
         if( this->bNativePresent && mad==1 )
             fprintf(fp, "%s\n", "native");
     }
     if( eType==MAD &&  mad==1 && !this->bMad1Auto )
     {
         fprintf(fp, "%s ", "normalize");
         for( j=0; j<nWaves; j++ ) 
         {
            if( bMad1Waves[j] )
                fprintf(fp, "%d ", j+1);
         }
         fprintf(fp, "\n");
     }
     if( dMad1Maxres>0.0 && mad==1 )
         fprintf(fp, "filter 2 %8.3f\n", dMad1Maxres);
     if( mad==1 && eType!=MOLREP && eType!=ABI) 
     {
        if( nMad1Refls>0 )
            fprintf(fp, "%s %d\n", "nref", nMad1Refls);
     }

     if( eType==MOLREP )
     {
         fprintf(fp, "%%remo\n");
//300409         if( !bMRUsePseudo ) fprintf(fp, "nopseudo\n");
         if( bMRUsePseudo ) fprintf(fp, "pseudo\n");
         if (bMRRes)
             fprintf(fp, "range   %.2f  %.2f\n", dMRMinres, dMRMaxres);
//         else
//             fprintf(fp, "range   0  0\n");
         if(!bMRTSAuto)
             fprintf(fp, "Threshold   %.2f  %.2f  %.2f %.2f\n",
                    dMRTSRot, dMRTSRotLoc, dMRTSTra, dMRTSTraLoc);

         for(i=0; i<nMRModels; i++)
         {

             fprintf(fp, "model\n");
             if( bMRPolyalanine[i])
                 fprintf(fp, "Polyalanine\n");
             if( bMRCutModel[i])
                 fprintf(fp, "cut %d %d\n", n1MR[i], n2MR[i]);
             const char *het = bMRUseHetero[i] ? "hetam" : " ";
             fprintf(fp, "fragment  %s %s\n", het, fromUTF8(szMRFile[i]).c_str());
             if( bMREdm[i] ) fprintf(fp, "refin\n");
             if(bMRRot[i])
                fprintf(fp, "RotSelected %d %d %d\n", iMRalfa[i], iMRbeta[i], iMRgamma[i]);
             if(bMRTrans[i])
                fprintf(fp, "TranSelected %f %f %f\n", dMRxx[i], dMRyy[i], dMRzz[i]);
             fprintf(fp, "monomers   %d\n", nMRNmol[i]);
             if(cMRAdp[i] == 'P') 
                 fprintf(fp, "BFactor PDB\n"); 
             else if(cMRAdp[i] == 'W') 
                 fprintf(fp, "BFactor Wilson\n"); 
             else
                 fprintf(fp, "BFactor 20\n"); 
             if(bMRSeq[i])
                 fprintf(fp, "idseq %lf\n", dMRvalue[i]);
             else
                 fprintf(fp, "rms %lf\n", dMRvalue[i]);

             fprintf(fp, "endmodel\n");
         }
     }
     if( eType==ABI ) 
     {
         if(!Ilm::PTheProject->IsRecover())
         {
            fprintf(fp, "%%phase\n");
            if( bAbiPatterson ) 
            {
              fprintf(fp, "Patterson ");
              if(!bAbiDefaults)
              {
                 for(int i=0; i< nAbiBlocks; i++ ) 
                     fprintf(fp, " %d %d",nAbiTrialG[i],nAbiTrialS[i]);
              }
              fprintf(fp, "\n");
            }
            else 
            {
              fprintf(fp, "Tangent ");
              if(!bAbiDefaults)
              {
//080309                 fprintf(fp, "tangent ");
                 for(int i=0; i< nAbiBlocks; i++ ) 
                     fprintf(fp, " %d %d",nAbiTrialG[i],nAbiTrialS[i]);
//080309                 fprintf(fp, "\n");
              }
              fprintf(fp, "\n");
            }
            if(iEdmFreeLunch && (dresMax > dresExtramin))
                 fprintf(fp, "FreeLunch\n");
         }
         else
         {
            fprintf(fp, "%%recover\n");
            fprintf(fp, "  MTZ %s \n", fromUTF8(Ilm::PTheProject->GetRecoverMTZ()).c_str());
            fprintf(fp, "  Label %s \n", Ilm::PTheProject->GetRecoverMTZLab().c_str());
            int ntrial = *((int *) get_nth_field(Ilm::PTheProject->GetRecoverData().c_str(), " ", 1, 'i'));
            float fom = *((float *) get_nth_field(Ilm::PTheProject->GetRecoverData().c_str(), " ", 2, 'f'));
            fprintf(fp, "  Trial %d %f\n", ntrial, fom);
         }
     }

     if( mad==1 && eType!=MOLREP && eType!=ABI) 
     {
           fprintf(fp, "%%phase\n");
           if( bMad1Patterson ) 
           {
               fprintf(fp, "Patterson ");
               if( !bMad1Defaults ) fprintf(fp, " %d %d", nMad1Trials1, nMad1Trials2);
               fprintf(fp, "\n");
           } 
           else 
           {
               fprintf(fp, "Tangent ");
               if( !bMad1Defaults ) fprintf(fp, "  %d %d", nMad1Trials1, nMad1Trials2);
               fprintf(fp, "\n");
           }
           if( bMad1MorePower ) 
               fprintf(fp, "size L \n");
     }
     if( mad==3 ) 
     {
        int nNum;
        if((eType == MIR) || (eType == MRS))
            nNum = nDeriv;
        else
            nNum = nWaves;

        for( j=0; j<nNum; j++ )
        {
             if( !bProDerivs[j] )
               continue;
             fprintf(fp, "%%invar\n");
             fprintf(fp, "derivative %s\n", szMirElement[j][0] );
             fprintf(fp, "%%phase\n");
             fprintf(fp, "maxt   %d %d\n", nProTrials, nProStore);
             fprintf(fp, "%%%s\n", "extend");
             if( eType==MRS )
               fprintf(fp, "adm     ");
             else
               fprintf(fp, "dm    ");
             fprintf(fp, "%s \n", szMirElement[j][0]);
             fprintf(fp, "set   1 %d\n", nProUse);
          }

          if( eType==MIR ) 
          {
            fprintf(fp, "%%extend\nmir\n");
          }
          else if( eType==MRS ) 
          {
            fprintf(fp, "%%extend\nmas\n");
          }
          fprintf(fp, "position %s\n", fromUTF8(GetFilename(PROFLEXFRAC_EXT)).c_str() );
          fprintf(fp, "set   1 %d\n", nProUse);
          if(bEdm)
          {
             std::stringstream strm1;
             AddFlex(strm1);
             fprintf(fp, "%s", strm1.str().c_str());
          }
     }
     if( mad==2 ) 
     {
         fprintf(fp, "%%%s\n", "extend");
         if( eType==MAD ) 
         {
             fprintf(fp, "%s ", "mad");
             for( j=0; j<nWaves; j++ ) 
             {
                  if( bMad2Waves[j] )
                      fprintf(fp, "%d ", j+1);
             }
             fprintf(fp, "\n");
         }
         else if( eType==MIR ) 
         {
             fprintf(fp, "%s", "mir  ");
             for( j=0; j<nDeriv; j++ ) {
                 if( bMad2Waves[j] )
                    fprintf(fp, "%s ", szMirElement[j][0]);
             }
             fprintf(fp, "\n");
         }
         else if( eType==MRS ) 
         {
             fprintf(fp, "%s", "mas  ");
             for( j=0; j<nDeriv; j++ ) {
                if( bMad2Waves[j] )
                    fprintf(fp, "%s ", szMirElement[j][0]);
             }
             fprintf(fp, "\n");
         }
         if(!gext::str_is_white(szMad2File))
             fprintf(fp, "%s %s\n", "position", fromUTF8(szMad2File).c_str());
         else 
         {
             fprintf(fp, "%s %s\n", "position", fromUTF8(GetFilename(Ilm::MAD1FRAC_EXT)).c_str());
             fprintf(fp, "%s %d %d\n", "set", iMad2SetMin, iMad2SetMax);
         }
         if( bMad2Lsq )
         {
             int nx = 6, nw =0;
             if( bMad2LsqAdp )
                 nx = 1;
             if( bMad2LsqPos )
                 nx = 2;
             if( bMad2LsqSof )
                 nx = 4;
             if( bMad2LsqPos && bMad2LsqAdp )
                 nx = 3;
             if( bMad2LsqPos && bMad2LsqSof && bMad2LsqAdp )
                 nx = 5;
             if( bMad2LsqF2 )
                 nw = 1;
             if( bMad2LsqF1 )
                 nw = 2;
             if( bMad2LsqF1 && bMad2LsqF2 )
                 nw = 3;
             if( nx==6 && nw==0 )
                 nx=5, nw=3;
             if( eType==MIR )
                 nw = 0;
             fprintf(fp, "%s %d%d\n", "lsq", nx, nw);
         }
         else
             fprintf(fp, "%s 0\n", "lsq");
         if(bEdm)
         {
            std::stringstream strm1;
            AddFlex(strm1);
            fprintf(fp, "%s", strm1.str().c_str());
         }
    }
    
    if(eType == ABI)
    {
         if(bEdm)
         {
            std::stringstream strm1;
            AddFlex(strm1);
            fprintf(fp, "%s", strm1.str().c_str());
         }
//       fprintf(fp, "%%edm\n");
//       fprintf(fp, "   dm 1 1 0 0\n");
//       fprintf(fp, "   convc -1\n");
//       fprintf(fp, "   refextension 0\n");
    }

    fprintf(fp, "%%end\n");
    fclose(fp);
    return TRUE;
  }

  //! Writes instructions for Dea
  void Project::SaveAsDea() const
  {
    FILE *fp = NULL;
    char *str =  g_build_filename(szWorkDir, "dea.sir", NULL);
    char *strmtz =  g_build_filename(szWorkDir, "dea_amb.mtz", NULL);
    fp = fOpen(str, "w");

    fprintf(fp, "%%nowindow\n");
    fprintf(fp, "%%structure dea_edm\n");
    fprintf(fp, "%%initialize\n");
    fprintf(fp, "%%%s %s\n", "job", szJobTitle);
    fprintf(fp, "%%%s\n", "data");
    fprintf(fp, "mtz %s\n", strmtz);
//    fprintf(fp, "mtz dea_amb.mtz\n");
//    fprintf(fp, "label H K L F_ILM SIGF_ILM PHIC FOM FC FreeR_flag \n");
    fprintf(fp, "label H K L F_ILM SIGF_ILM FreeR_flag PHIC FOM FC \n");
     switch(iCont)
     {
     case 0:               // Molecular Weight
         fprintf(fp, "Weight %lf\n", dContent);
       break;
     case 1:               // Number of residues
         fprintf(fp, "nResidues %d\n", iround(dContent));
       break;
     case 2:               // Sequence
         fprintf(fp, "Sequence %s\n", fromUTF8(szSeqFile).c_str());
       break;
     }
    fprintf(fp, "nMolecules %d\n", nMolecules);
    fprintf(fp, "phase 3\n");
    fprintf(fp, "%%edm\n");
    fprintf(fp, "dm 1 1 0 0\n");
    fprintf(fp, "partial 2 0 0\n");
    fprintf(fp, "%%end\n");

    fclose(fp);
  }

  void Project::CreateCadShell() const
  {
    FILE *fp = NULL;
    char *str =  NULL;

// Crea il file cad_obs.sh
    char *effe  = (char *)get_nth_field(szSelectedColumns, " ", 4, 's');
    char *sigma = (char *)get_nth_field(szSelectedColumns, " ", 5, 's');
    str = g_build_filename(szWorkDir, CADMD_SH, NULL);
    fp = fOpen(str, "w");
#ifndef _WIN32
    fprintf(fp, "#!/bin/sh\n");
#endif
    fprintf(fp, "cad HKLIN1 %s HKLOUT dea_obs1.mtz < %s\n", fromUTF8(szMtzFile).c_str(), CADMD_INP);
    fprintf(fp, "freerflag HKLIN dea_obs1.mtz HKLOUT dea_obs.mtz <%s \n", FREEMD_INP);
    fclose(fp);
    ChangeMode(str, tXt);
    g_free(str);
    str = g_build_filename(szWorkDir, CADMD_INP, NULL);
    fp = fOpen(str, "w");
    fprintf(fp, "LABIN FILE 1 E1=%s  E2=%s\n", effe, sigma);
    fprintf(fp, "LABOUT FILE 1 E1=F_ILM  E2=SIGF_ILM\n");
    fprintf(fp, "END\n");
    fclose(fp);
    g_free(str);
    str = g_build_filename(szWorkDir, FREEMD_INP, NULL);
    fp = fOpen(str, "w");
    fprintf(fp, "END\n");
    fclose(fp);
    g_free(str);
#if 0
    fprintf(fp, "cad HKLIN1 %s HKLOUT dea_obs1.mtz << END-CAD\n", fromUTF8(szMtzFile).c_str());
    fprintf(fp, "LABIN FILE 1 E1=%s  E2=%s\n", effe, sigma);
    fprintf(fp, "LABOUT FILE 1 E1=F_ILM  E2=SIGF_ILM\n");
    fprintf(fp, "END\n");
    fprintf(fp, "END-CAD\n");
    fprintf(fp, "freerflag HKLIN dea_obs1.mtz HKLOUT dea_obs.mtz <<eof-freerflag\n");
    fprintf(fp, "END\n");
    fprintf(fp, "eof-freerflag\n");
    fclose(fp);
    ChangeMode(str, tXt);
#endif
    if(!bModel)
    {
       effe  = (char *)get_nth_field(szSelectedColumnsPh, " ", 4, 's');
       sigma = (char *)get_nth_field(szSelectedColumnsPh, " ", 5, 's');
       char *ph  = (char *)get_nth_field(szSelectedColumnsPh, " ", 6, 's');
       char *we = (char *)get_nth_field(szSelectedColumnsPh, " ", 7, 's');
       str = g_build_filename(szWorkDir, CADPH_SH, NULL);
       fp = fOpen(str, "w");
#ifndef _WIN32
       fprintf(fp, "#!/bin/sh\n");
#endif
       fprintf(fp, "cad HKLIN1 %s HKLOUT dea_%s_dedm_edm01.mtz < %s\n", fromUTF8(szMtzFilePh).c_str(), ambpre[nBuildPgm], CADPH_INP);
       fprintf(fp, "freerflag HKLIN dea_%s_dedm_edm01.mtz HKLOUT dea_%s_dedm_edm0.mtz <%s\n", ambpre[nBuildPgm], ambpre[nBuildPgm], FREEPH_INP);
       fclose(fp);
       ChangeMode(str, tXt);
       g_free(str);
       str = g_build_filename(szWorkDir, CADPH_INP, NULL);
       fp = fOpen(str, "w");
       fprintf(fp, "LABIN FILE 1 E1=%s  E2=%s E3=%s E4=%s\n", effe, sigma, ph, we);
       fprintf(fp, "LABOUT FILE 1 E1=F  E2=SIGF E3=PHI_EDM  E4=WT_EDM\n");
       fprintf(fp, "END\n");
       fclose(fp);
       g_free(str);
       str = g_build_filename(szWorkDir, FREEPH_INP, NULL);
       fp = fOpen(str, "w");
       fprintf(fp, "END\n");
       fclose(fp);
#if 0
       fprintf(fp, "cad HKLIN1 %s HKLOUT dea_%s_dedm_edm01.mtz << END-CAD\n", fromUTF8(szMtzFilePh).c_str(), ambpre[nBuildPgm]);
       fprintf(fp, "LABIN FILE 1 E1=%s  E2=%s E3=%s E4=%s\n", effe, sigma, ph, we);
       fprintf(fp, "LABOUT FILE 1 E1=F  E2=SIGF E3=PHI_EDM  E4=WT_EDM\n");
       fprintf(fp, "END\n");
       fprintf(fp, "END-CAD\n");
       fprintf(fp, "freerflag HKLIN dea_%s_dedm_edm01.mtz HKLOUT dea_%s_dedm_edm0.mtz <<eof-freerflag\n", ambpre[nBuildPgm], ambpre[nBuildPgm]);
       fprintf(fp, "END\n");
       fprintf(fp, "eof-freerflag\n");
       fclose(fp);
       ChangeMode(str, tXt);
#endif
    }

  }

  void Project::CreateScriptsPhenix() const
  {
    FILE *fp = NULL;
    char *str =  NULL;

// Crea il file phenixrun_model.sh (solo nel caso "Starting from model")
    if(bModel)
    {
       str = g_build_filename(szWorkDir, PHEMD_SH, NULL);
       fp = fOpen(str, "w");

#ifndef _WIN32
       fprintf(fp, "#!/bin/sh\n");
#endif
       fprintf(fp, "echo \"Running AutoBuild on %s data...\"\n", szStructName);
       fprintf(fp, "phenix.autobuild  write_run_directory_to_file=dea_phenix0.log \\\n");
       if(iCont == 2)
          fprintf(fp, "   seq_file=%s \\\n", fromUTF8(szSeqFile).c_str());
       fprintf(fp, "   input_data_file=dea_obs.mtz \\\n");
       fprintf(fp, "   input_labels=\"F_ILM SIGF_ILM None None None None None None FreeR_flag\" \\\n");
       fprintf(fp, "   input_pdb_file=%s \\\n",  fromUTF8(szMRFile[0]).c_str());
       fprintf(fp, "   ncs_copies=%d \\\n", nMolecules);
       fprintf(fp, "   nproc=%d\n", nCPU());
       fclose(fp);
       ChangeMode(str, tXt);
    }
// Crea il file phenixrun_phases.sh (in entrambi i cas)
//    else
    {
       str = g_build_filename(szWorkDir, PHEPH_SH, NULL);
       fp = fOpen(str, "w");
   
#ifndef _WIN32
       fprintf(fp, "#!/bin/sh\n");
#endif
       fprintf(fp, "echo \"Running AutoBuild on %s data...\"\n", szStructName);
       fprintf(fp, "phenix.autobuild  write_run_directory_to_file=dea_phenix$1.log \\\n");
       if(iCont == 2)
          fprintf(fp, "   seq_file=%s \\\n", fromUTF8(szSeqFile).c_str());
       fprintf(fp, "   input_data_file=dea_phenix_dedm_edm$2.mtz \\\n");
       fprintf(fp, "   input_labels=\"F SIGF PHI_EDM WT_EDM None None None None FreeR_flag\" \\\n");
       if( !gext::str_is_white(szMRFile[1]) )
           fprintf(fp, "   input_ha_file=%s \\\n", fromUTF8(szMRFile[1]).c_str());
       fprintf(fp, "   ncs_copies=%d \\\n", nMolecules);
       fprintf(fp, "   nproc=%d\n", nCPU());
       fclose(fp);
       ChangeMode(str, tXt);
    }
  }

  void Project::CreateScriptsArp() const
  {
    FILE *fp = NULL;
    char *str =  NULL;

// Crea il file arprun_model.sh (solo nel caso "Starting from model")
    if(bModel)
    {
       str = g_build_filename(szWorkDir, ARPMD_SH, NULL);
       fp = fOpen(str, "w");

#ifndef _WIN32
       fprintf(fp, "#!/bin/sh\n");
#endif
       fprintf(fp, "echo \"Running ARP/wARP AutoBuilding on %s data...\"\n", szStructName);
       fprintf(fp, "auto_tracing.sh \\\n");
       fprintf(fp, "   datafile %s \\\n", fromUTF8(g_build_filename(szWorkDir, "dea_obs.mtz", NULL)).c_str());
       fprintf(fp, "   residues %d \\\n", (int) (dContent*nMolecules));
       fprintf(fp, "   workdir %s \\\n", fromUTF8(szWorkDir).c_str());
       fprintf(fp, "   fp F_ILM sigfp SIGF_ILM  freelabin FreeR_flag \\\n");
       fprintf(fp, "   modelin %s \\\n",  fromUTF8(szMRFile[0]).c_str());
       if(iCont == 2)
          fprintf(fp, "   seqin %s \\\n", fromUTF8(GetFilename(".pir")).c_str());
       fprintf(fp, "   cgr %d \n", nMolecules);
       fclose(fp);
       ChangeMode(str, tXt);
    }
// Crea il file arprun_phases.sh (in entrambi i cas)
//    else
    {
       str = g_build_filename(szWorkDir, ARPPH_SH, NULL);
       fp = fOpen(str, "w");
   
#ifndef _WIN32
       fprintf(fp, "#!/bin/sh\n");
#endif
       fprintf(fp, "echo \"Running ARP/wARP AutoBuilding on %s data...\"\n", szStructName);
       fprintf(fp, "auto_tracing.sh \\\n");
       fprintf(fp, "   datafile %s \\\n", fromUTF8(g_build_filename(szWorkDir, "dea_arp_dedm_edm$2.mtz", NULL)).c_str());
       fprintf(fp, "   residues %d \\\n", (int) (dContent*nMolecules));
       fprintf(fp, "   workdir %s \\\n", fromUTF8(szWorkDir).c_str());
       fprintf(fp, "   fp F sigfp SIGF phibest PHI_EDM fom WT_EDM freelabin FreeR_flag \\\n");
       if(iCont == 2)
          fprintf(fp, "   seqin %s \\\n", fromUTF8(GetFilename(".pir")).c_str());
       fprintf(fp, "   cgr %d \n", nMolecules);
       fclose(fp);
       ChangeMode(str, tXt);
    }
  }

  void Project::CreateScriptsBuccNau() const
  {
    char *str =  NULL;
    std::ofstream of;

    std::string pathCCP4(" ");
    std::string pathPython(" ");
    const char *ambccp4 = Ambiente("CCP4_MASTER");
    if(ambccp4 != NULL)
    {
       const char *path = Ambiente("CBIN");
       if(path != NULL)
          pathCCP4.assign(path);
       char *tpath = find_program_in_path("python");
       if(tpath == NULL)
       {
          tpath = g_strdup(FindProg("python*", ambccp4));
          if(tpath)
          {
             pathPython.assign(g_build_filename (ambccp4, tpath, "python", NULL));
             g_free(tpath);
          }
       }
       else
       {
          pathPython.assign(tpath);
          g_free(tpath);
       }
    }
      
    if(bModel)
    {
       if(nBuildPgm == 3)
          str = g_build_filename(szWorkDir, BUCMD_SH, NULL);
       else
          str = g_build_filename(szWorkDir, NAUMD_SH, NULL);
       of.open(str);
#ifndef _WIN32
       of << "#! /bin/bash\n";
#endif
       of << "echo \"Running " << ambp[nBuildPgm] << " AutoBuilding on "
          << szStructName << "data...\"\n" 
          << pathPython << " "
          << g_build_filename(pathCCP4.c_str(), amb_prg[nBuildPgm], NULL) 
          << " " << CONT_CHAR << "\n";
       if(iCont == 2)
          of << "   -seqin " 
             << fromUTF8(GetFilename(".pir")) << " " << CONT_CHAR << "\n";
       of << "   -mtzin " 
          << fromUTF8(g_build_filename(szWorkDir, "dea_obs.mtz", NULL))
          << " " << CONT_CHAR << "\n"
          << "   -pdbin " << fromUTF8(szMRFile[0]) << " " << CONT_CHAR << "\n"
          << "   -colin-fo     F_ILM,SIGF_ILM " << CONT_CHAR << "\n"
          << "   -colin-free   FreeR_flag\n";
       of.close();
       ChangeMode(str, tXt);
    }
    g_free(str);
    if(nBuildPgm == 3)
       str = g_build_filename(szWorkDir, BUCPH_SH, NULL);
    else
       str = g_build_filename(szWorkDir, NAUPH_SH, NULL);
    of.open(str);
    std::string nome("dea_");
    nome.append(ambpre[nBuildPgm]);
#ifdef _WIN32
    nome.append("_dedm_edm%2.mtz");
#else
    nome.append("_dedm_edm$2.mtz");
#endif
#ifndef _WIN32
    of << "#! /bin/bash\n";
#endif
    of << "echo \"Running " << ambp[nBuildPgm] << " AutoBuilding on "
       << szStructName << "data...\"\n" 
       << pathPython << " "
       << g_build_filename(pathCCP4.c_str(), amb_prg[nBuildPgm], NULL) 
       << " " << CONT_CHAR << "\n";
    if(iCont == 2)
       of << "   -seqin " 
          << fromUTF8(GetFilename(".pir")) << " " << CONT_CHAR << "\n";

    of << "   -mtzin " 
       << fromUTF8(g_build_filename(szWorkDir, nome.c_str(), NULL))
       << " " << CONT_CHAR << "\n"
       << "   -colin-fo     F,SIGF " << CONT_CHAR << "\n"
       << "   -colin-phifom PHI_EDM,WT_EDM " << CONT_CHAR << "\n"
       << "   -colin-free   FreeR_flag\n";
    of.close();
    ChangeMode(str, tXt);
    g_free(str);
  }

#if 0
  void Project::CreateScriptsBuccNau() const
  {
    FILE *fp = NULL;

    if(bModel)
    {
       if(nBuildPgm == 3)
          str = g_build_filename(szWorkDir, BUCMD_SH, NULL);
       else
          str = g_build_filename(szWorkDir, NAUMD_SH, NULL);
       fp = fOpen(str, "w");

#ifndef _WIN32
       fprintf(fp, "#!/bin/sh\n");
#endif
       fprintf(fp, "echo \"Running %s AutoBuilding on %s data...\"\n", ambp[nBuildPgm], szStructName);
       fprintf(fp, "%s %s\n", amb_prg[nBuildPgm], CONT_CHAR);
       if(iCont == 2)
          fprintf(fp, "   -seqin %s %s\n", fromUTF8(GetFilename(".pir")).c_str(), CONT_CHAR);
       //fprintf(fp, "   residues %d %s \n", (int) (dContent*nMolecules), CONT_CHAR);
       //fprintf(fp, "   cgr %d %s \n", nMolecules, CONT_CHAR);
       fprintf(fp, "   -mtzin %s %s\n", fromUTF8(g_build_filename(szWorkDir, "dea_obs.mtz", NULL)).c_str(), CONT_CHAR);
       fprintf(fp, "   -pdbin %s %s\n", fromUTF8(szMRFile[0]).c_str(), CONT_CHAR);
       fprintf(fp, "   -colin-fo     F_ILM,SIGF_ILM %s\n", CONT_CHAR);
       fprintf(fp, "   -colin-free   FreeR_flag\n");
       fclose(fp);
       ChangeMode(str, tXt);
    }
    if(nBuildPgm == 3)
       str = g_build_filename(szWorkDir, BUCPH_SH, NULL);
    else
       str = g_build_filename(szWorkDir, NAUPH_SH, NULL);
    fp = fOpen(str, "w");
    std::string nome("dea_");
    nome.append(ambpre[nBuildPgm]);
    nome.append("_dedm_edm$2.mtz");

#ifndef _WIN32
       fprintf(fp, "#!/bin/sh\n");
#endif
    fprintf(fp, "echo \"Running %s AutoBuilding on %s data...\"\n", ambp[nBuildPgm], szStructName);
    fprintf(fp, "%s %s\n", amb_prg[nBuildPgm], CONT_CHAR);
    if(iCont == 2)
       fprintf(fp, "   -seqin %s %s\n", fromUTF8(GetFilename(".pir")).c_str(), CONT_CHAR);
    //fprintf(fp, "   residues %d %s \n", (int) (dContent*nMolecules), CONT_CHAR);
    //fprintf(fp, "   cgr %d %s \n", nMolecules, CONT_CHAR);
    fprintf(fp, "   -mtzin %s %s\n", fromUTF8(g_build_filename(szWorkDir, nome.c_str(), NULL)).c_str(), CONT_CHAR);
    fprintf(fp, "   -colin-fo     F,SIGF %s\n", CONT_CHAR);
    fprintf(fp, "   -colin-phifom PHI_EDM,WT_EDM %s\n", CONT_CHAR);
    fprintf(fp, "   -colin-free   FreeR_flag\n");
    fclose(fp);
    ChangeMode(str, tXt);
  }

  void Project::CreateScriptsNautilus() const
  {
       CreateScriptsBuccaneer();
  }
#endif

  //! Returns command for Task1
  std::string Project::Mad1Cmd() const
  {
    int tipo = 1;
    if(eType > EDM)
       tipo = eType - 1;
#ifdef _WIN32
    return "\'" + TheApp.Executables()[IExeSir] + "' '" + GetFilename2(INP_EXT, tipo) + "' '" + GetFilename2(OUT_EXT, tipo) + "\'";
#else
    return TheApp.Executables()[IExeSir] + " " + GetFilename2(INP_EXT, tipo) + " " + GetFilename2(OUT_EXT, tipo);
#endif
  }

  //! Returns command for Task2
  std::string Project::Mad2Cmd() const
  {
#ifdef _WIN32
    gchar *tmp;
    std::string r;
    tmp = g_shell_quote(TheApp.Executables()[IExeSirProt].c_str());
    r = tmp;
    g_free(tmp);
    r += " ";
    tmp = g_shell_quote(bPro?GetFilename2(INP_EXT, 0).c_str():GetFilename2(INP_EXT, 2).c_str());
    r += tmp;
    g_free(tmp);
    r += " ";
    tmp = g_shell_quote(bPro?GetFilename2(OUT_EXT, 0).c_str():GetFilename2(OUT_EXT, 2).c_str());
    r += tmp;
    g_free(tmp);
    return r;
#else
    return TheApp.Executables()[IExeSirProt] + " " 
      + (bPro?GetFilename2(INP_EXT, 0):GetFilename2(INP_EXT, 2))  + "  " + (bPro?GetFilename2(OUT_EXT, 0):GetFilename2(OUT_EXT, 2));
#endif
  }

  std::string Project::EDMCmd() const
  {
#ifdef _WIN32
    gchar *tmp;
    std::string r;
    tmp = g_shell_quote(TheApp.Executables()[IExeSirProt].c_str());
    r = tmp;
    g_free(tmp);
    r += " ";
    //tmp = g_shell_quote(TheApp.SIRProtConfigDir().c_str());
    //r += tmp;
    //g_free(tmp);
    //r += " ";
    tmp = g_shell_quote(GetFilename2(INP_EXT, EDM-1).c_str());
    r += tmp;
    g_free(tmp);
    r += " ";
    tmp = g_shell_quote(GetFilename2(OUT_EXT, EDM-1).c_str());
    r += tmp;
    g_free(tmp);
    return r;
#else
    return TheApp.Executables()[IExeSirProt] + " "
      + GetFilename2(INP_EXT, EDM-1) + "  " + GetFilename2(OUT_EXT, EDM-1);
#endif
  }

  std::string Project::DEACmd() const
  {
     std::stringstream os ;
     os.str("");
     os << "To run the DEA project, please check in the '"
        << szWorkDir << "'\ndirectory for the script dea_run.sh.\n"
        << "To change the " << ambp[nBuildPgm] << " condition, please check the script";
       if(bModel)
       {
          os << "s";
          if(nBuildPgm == 1)
             os << " " << ARPMD_SH << " and";
          else if(nBuildPgm == 2)
             os << " " << PHEMD_SH << " and";
          else if(nBuildPgm == 3)
             os << " " << BUCMD_SH << " and";
          else if(nBuildPgm == 4)
             os << " " << NAUMD_SH << " and";
       }
       if(nBuildPgm == 1)
          os << " " << ARPPH_SH << ".\n";
       else if(nBuildPgm == 2)
          os << " " << PHEPH_SH << ".\n";
       else if(nBuildPgm == 3)
          os << " " << BUCPH_SH << ".\n";
       else if(nBuildPgm == 4)
          os << " " << NAUPH_SH << ".\n";

     return os.str();
  }

  //! Returns full pathname for the file project basename and suffix suffix
  std::string Project::GetFilename(const gchar *suffix) const
  {
    std::string r(szStructName);
    gchar *fn = NULL;

    r += suffix;
    fn = g_build_filename(szWorkDir, r.c_str(), NULL);
    r = fn;
    g_free(fn);
    return r;
  }

  std::string Project::GetFilename2(const gchar *suffix, int ind) const
  {
    gchar *fn = NULL;
    if(ind == (DEA - 1))
    {
       std::string t("dea");

       t += suffix;
       fn = g_build_filename(szWorkDir, t.c_str(), NULL);
       t = fn;
       g_free(fn);
       return t;
    }
    else if((strcmp(suffix, LOCK_EXT) == 0) && (ind == 1))
    {
       std::string s(PACKAGE);
       s += g_strdup_printf("%d", ind);
       s += suffix;
       fn = g_build_filename(TheApp.ConfDir().c_str(),s.c_str(), NULL);
       s = fn;
       g_free(fn);
       return s;
    }
    else
    {
       std::string r(szStructName);

       r += TYPE_EXT[ind];
       r += suffix;
       fn = g_build_filename(szWorkDir, r.c_str(), NULL);
       r = fn;
       g_free(fn);
       return r;
    }
  }

  void Project::_ValidateGen(std::ostream &ostr ) // const
  {
    //ostr = "";
    if( !g_file_test(szWorkDir, G_FILE_TEST_IS_DIR) )
      throw UserError("Working directory does not exist");

    if( gext::str_is_white(szJobTitle) )
      throw UserError("Provide job title");

    if( gext::str_is_white(szStructName) )
      throw UserError("Provide structure title");

    ostr << PACKAGE " Project file: " << GetFilename(PROJECT_EXT) << "\n";
  }

 int Project::AnalizzaSequenza(const std::string &buf) const
 {
     const std::string Acids = "ACEGPRTUXYZ";
     for(unsigned int i=0; i<strlen(buf.c_str()); i++)
     {
         char seqchar = buf.c_str()[i];
         if(isupper(seqchar))
         {
            bool found = false;
            for(unsigned int j=0; j<strlen(Acids.c_str()); j++)
            {
                char acid = Acids.c_str()[j];
                if(seqchar == acid)
                {
                   found = true;
                   break;
                }
            }
            if(!found)
               return 1;
                
         }
      }
      return 2;
 }

 int Project::ContentFromSequence() const
 {
      std::string fn = szSeqFile;
      int nResidui = 0;  
      int NumSpec[6] = {0, 0, 0, 0, 0, 0};
      bool errore = false;

      if(!fn.empty())
      {
         FILE *fp = fOpen(szSeqFile, "r");
         if( fp)
         {
            gchar *buf = g_new0(gchar, MaxLen);
            if(fgets(buf, MaxLen, fp))
            {
               if(buf[0] != '>')
                  errore = true;
            }
            else
               errore = true;
            while(fgets(buf, MaxLen, fp) && !errore)
            {
               if(gext::str_is_white(buf))
                  errore = true;
               else if(buf[0] != '>')
               {
                  std::string buff = buf;
                  int nResLoc = 0;
                  GString *gseq = g_string_new(buff.c_str());
                  const std::string seque = g_string_ascii_up(gseq)->str;

                  int ifRSeq = AnalizzaSequenza(seque);
                  for(unsigned int i=0; i<strlen(seque.c_str()); i++)
                  {
                      char seqchar = seque.c_str()[i];
                      if(isupper(seqchar))
                      {
                         nResidui++;
                         nResLoc++;
                         int j = seqchar - 'A';
                         if ( (seqchar == 'X') && (ifRSeq == 2) )
                            continue;
                         if ( (seqchar == 'X') && (ifRSeq == 1) )
                            j = 0;
                         if(ifRSeq == 1)
                         {
                             for(int k=1; k<5; k++)
                                 NumSpec[k] += RES_C[j][k-1];
                         }
                         else
                         {
                             for(int k=0; k<4; k++)
                                 NumSpec[k] += AMI_C[j][k];
                             NumSpec[5] += AMI_C[j][4];
                         }
                      }
                      else if( (seqchar == ' ') || (seqchar == '-') )
                         ;
                      else if( (seqchar == '*') && (ifRSeq == 1) )
                         ;
                      else if( (seqchar == '\n') || (seqchar == '\r') )
                         ;
                      else
                         errore = true;
                  }
                  if(ifRSeq == 1)
                     NumSpec[0] += iround(nResLoc * DHPerResidue);
               }
            }
            fclose(fp);
            g_free(buf);
         }
      }
      if(errore)
         return -1;
      for(int n=0; n<6; n++)
          nPerResidue[n] = NumSpec[n]*nMolecules*nSpgrOrder;
      return nResidui;
}

int Project::ContentFromSequenceStr(const std::string &sequence) const
{
      FILE *fp = fOpen(szSeqFile, "r");
      bool Empty = true;
      std::string buffer = "";

      if(fp)
      {
          gchar *buf = g_new0(gchar, MaxLen);
          while(fgets(buf, MaxLen, fp))
          {
              buffer += buf;
              Empty = false;
          }
          g_free(buf);
          (void) fclose(fp);
      }

      gboolean result = Empty;      // il file e' pieno?
      if(!Empty)
      {
         // controlla che la sequenza non sia stata modificata
         if(buffer.compare(sequence) != 0)
         {
           std::string fn = szSeqFile;
           std::string mesg = "File " + fn + " already exists.\n"
            "Do you want to replace it?";
           result = gtkext::dlg_yesno(Ilm::PTheProject->MainWindow(), mesg.c_str());
         }
      }
      if(result)     // la sequenza e' stata introdotta o modificata via gui
      {
         if(!gext::str_is_white(sequence.c_str()))
         {
            fp = fOpen(szSeqFile, "w");
            if(fp)
            {
               (void) fputs(sequence.c_str(), fp);
               (void) fclose(fp);
            }
         }
      }
     else
         Ilm::PTheProject->ReadSequence(); //allinea file e gui

     return ContentFromSequence();  // legge dal file
}

  void Project::format_pir(std::string &s, std::string &t) const
  {
     int k;
     std::string s1; 
     t.clear();
     k = 0;
     for(int i=0; i<(int)s.length(); i+=10)
     {
         t.append(s.substr(i, 10));
         k += 10;
         if(k > 50)
         {
            t.append("\n");
            k = 0;
         }
         else
         {
            t.append(" ");
            k++;
         }
     }
  }

 void Project::FASTAToPIR() const
 {
      std::string fn = szSeqFile;
      std::string fn1 = GetFilename(".pir");
      bool inizio = true;

      if(!fn.empty())
      {
         FILE *fp  = fOpen(szSeqFile, "r");
         FILE *fp1 = fOpen(fn1.c_str(), "w");
         if( fp && fp1)
         {
            gchar *buf = g_new0(gchar, MaxLen);
            std::string buffer = "";
            std::string newbuf = "";
            while(fgets(buf, MaxLen, fp))
            {
               if(buf[0] == '>')
               {
                  if(inizio)
                     inizio = false;
                  else
                  {
                     format_pir(buffer,newbuf);
                     fprintf(fp1, "%s\n", newbuf.c_str());
                     buffer.erase();
                  } 
                  fprintf(fp1, ">---------+----------+----------+----------+----------+\n\n");
               }
               else
               {
                  std::string bufstr = buf;
                  buffer.append(filtrastr(bufstr, '\n'));
               }
            }
            format_pir(buffer,newbuf);
            fprintf(fp1, "%s\n", newbuf.c_str());
            fclose(fp);
            fclose(fp1);
            g_free(buf);
         }
      }
}

} // namespace Ilm
