MODULE anti_bump

USE RRESTR
USE elements

implicit none

real, parameter, private :: SIGMADEF = 0.3

type bump_settings
  integer :: baction = 0     ! 0 = bump disabled, 1 = default strategy, 2 = user strategy
  real    :: weight  = 1     ! weight for bump in the CF
  real    :: scaledist = 0.7 ! scale factor on distance
end type bump_settings

!S set_antibump_auto(bump, btable, atom, frag)                     !Generate automatically list of anti-bumping restraints
!F function bump_distance(z1,z2) result(dist)                      !Set target distances as sum of vdW radius
!S create_bump_table(atom, bumptab)                                !Create table of anti-bump restraints from the atomic model
!S activate_bump(bumptab,el1,el2,targ)                             !Activate bump restraint in the bump table and set target distance 
!S bump_from_table(bumptab,bump,atom,frag,bset)                    !Set restraints list from bump table
!S bump_from_string(string, atom, bump, btable, frag, scaled, ier) !Get anti-bump restraints from a string
!S set_antibump_spec(bump,atom,frag,spec,scaled,targ)              !Set anti-bump restraints for for species in the array spec
!S set_antibump_frag(bump,atom,frag1,frag2,scaled,targ)            !Set anti-bump restraint between 2 molecular fragments
!S set_antibump_spec1(bump,atom,frag,spec1,spec2,scaled,targ)      !Set all bump restraints between species spec1 spec2
!S print_antibump(kpr,bump,atom)                                   !Print list of anti-bumping restraints
!F antibump_function(bump,atom,doc,kpr) result(bfunc)              !Compute cost function for anti-bump restraints

CONTAINS

   subroutine set_antibump_auto(bump, btable, atom, frag, scaled)
!
!  Generate automatically list of anti-bumping restraints
!
   USE atom_type_util
   USE fragmentmod
   type(restraint_type), dimension(:), allocatable, intent(inout) :: bump
   type(restraint_type), dimension(:), allocatable, intent(inout) :: btable
   type(atom_type), dimension(:), allocatable, intent(in)         :: atom
   type(fragment_type), dimension(:), allocatable, intent(in)     :: frag
   real, intent(in)                                               :: scaled
   integer, dimension(4), parameter                               :: abelem = [C_at,N_at,O_at,S_at]
   integer                                                        :: i,j
!
   call set_antibump_spec(bump, atom, frag, eleminfo(abelem)%lab, scaled)
   if (nrestraints(btable) > 0) then  ! update the bump table
       do i=1,size(abelem)
          do j=i,size(abelem)
             call activate_bump(btable,abelem(i),abelem(j))
          enddo
       enddo
   endif
!
   end subroutine set_antibump_auto

!--------------------------------------------------------------

   real function bump_distance(z1,z2) result(dist)
!
!  Set target distances as sum of vdW radius
!
   integer, intent(in) :: z1,z2
   real, parameter :: FACTH = 0.9
!
   if ((z1 == O_at .and. z2 == O_at) .or.      &   ! riduce distance for possible h-bond
       (z1 == N_at .and. z2 == O_at) .or.      &
       (z1 == O_at .and. z2 == N_at)) then     
       dist = FACTH*(eleminfo(z1)%w_radius + eleminfo(z2)%w_radius)
   else
       dist = eleminfo(z1)%w_radius + eleminfo(z2)%w_radius
   endif
!
   end function bump_distance

!--------------------------------------------------------------

   subroutine create_bump_table(atom, bumptab)
!
!  Create table of anti-bump restraints from the atomic model
!
   USE progtype
   USE atom_type_util
   USE strutil
   USE realloc
   type(element_type), dimension(:), allocatable                :: elem
   type(atom_type), dimension(:), intent(in)                    :: atom
   type(restraint_type), dimension(:), allocatable, intent(out) :: bumptab
   integer                                                      :: nelem
   integer                                                      :: i,j
   integer                                                      :: nbump
!
   call elements_from_atom(atom,elem)
   nelem = numelem(elem)
   if (nelem > 0) then
       call reallocate_restr(bumptab,(nelem*nelem + nelem)/2)
       nbump = 0
       do i=1,nelem
          do j=i,nelem
             nbump = nbump + 1
             call reallocate(bumptab(nbump)%na,2)
             bumptab(nbump)%code = -4  ! negative value for not-active anti-bump
             bumptab(nbump)%na(1) = elem(i)%ptab 
             bumptab(nbump)%na(2) = elem(j)%ptab 
             bumptab(nbump)%targ = bump_distance(elem(i)%z, elem(j)%z)
!             bumptab(nbump)%targ = SCALEDIST*(elem(i)%w_radius + elem(j)%w_radius)
!              write(0,*)trim(elem(i)%lab)//'-'//trim(elem(j)%lab),bumptab(nbump)%targ,bump_distance(elem(i)%ptab,elem(j)%ptab)
          enddo
       enddo
   endif
!
   end subroutine create_bump_table

!--------------------------------------------------------------

   subroutine activate_bump(bumptab,el1,el2,targ)
!
!  Activate bump restraint in the bump table and set target distance 
!
   type(restraint_type), dimension(:), intent(inout) :: bumptab  
   integer, intent(in)                               :: el1,el2 ! pointers to table of elements
   real, intent(in), optional                        :: targ    ! target distance
   integer                                           :: i
!
   do i=1,size(bumptab)
      if ((bumptab(i)%na(1) == el1 .and. bumptab(i)%na(2) == el2) .or.   &
          (bumptab(i)%na(1) == el2 .and. bumptab(i)%na(2) == el1)) then
           bumptab(i)%code = abs(bumptab(i)%code)
           if (present(targ)) bumptab(i)%targ = targ
      endif
   enddo
!
   end subroutine activate_bump
   
!--------------------------------------------------------------

   subroutine bump_from_table(bumptab,bump,atom,frag,bset)
!
!  Set restraints list from bump table
!
   USE fragmentmod
   type(restraint_type), dimension(:), intent(in)                 :: bumptab
   type(restraint_type), dimension(:), allocatable, intent(inout) :: bump
   type(atom_type), dimension(:), allocatable, intent(in)         :: atom
   type(fragment_type), dimension(:), allocatable, intent(in)     :: frag
   type(bump_settings)                                            :: bset
   integer                                                        :: i
!
   do i=1,size(bumptab)
      if (bumptab(i)%code > 0) then
          call set_antibump_spec1(bump, atom, frag,   &
          eleminfo(bumptab(i)%na(1))%lab, eleminfo(bumptab(i)%na(2))%lab, bset%scaledist, bumptab(i)%targ)
      endif
   enddo
!
   end subroutine bump_from_table

!--------------------------------------------------------------

   subroutine bump_from_string(string, atom, bump, btable, frag, scaled, ier)
!
!  Get anti-bump restraints from a string
!
   USE strutil
   USE atom_basic, only:atom_type
   USE atom_type_util
   USE fragmentmod
   USE elements
   character(len=*), intent(in)                                   :: string
   type(atom_type), dimension(:), allocatable, intent(in)         :: atom
   type(restraint_type), dimension(:), allocatable, intent(inout) :: bump
   type(restraint_type), dimension(:), allocatable, intent(inout) :: btable
   type(fragment_type), dimension(:), allocatable, intent(in)     :: frag
   real, intent(in)                                               :: scaled
   integer, intent(out)                                           :: ier
   character(len=len_trim(string))                                :: line
   integer                                                        :: nspec
   character(len=20), dimension(4)                                :: wordv
   integer                                                        :: nword
   logical                                                        :: ok_bump 
   integer                                                        :: i
   real                                                           :: val, targ
   integer                                                        :: lenv
   integer                                                        :: irep
   integer, dimension(10)                                         :: ivet
   real, dimension(10)                                            :: vet
   integer                                                        :: iv
   integer                                                        :: nfrag
!
   ier = 0
   ok_bump = .true.
   line = trim(adjustl(string))
   call s_filter(line)
   if (len_trim(line) /= 0) then
       call s_rep(line,'frag',' ',irep)
       select case(irep) 
         case (0)    ! set bump as: bump specie1 specie2 target_distance
           call get_words(line,wordv,nword)
           if (nword >=2 ) then
               nspec = 0
               targ = -2
               do i=1,nword
                  if (nspec == 2) then
                      call s_to_r(wordv(i),val,ier,lenv)
                      if (ier == 0) targ = val
                      exit
                  else
                      if (numatomspec(atom,wordv(i)) > 0) then
                          nspec = nspec + 1
                      endif
                  endif
               enddo
               if (nspec == 2) then
                   if (targ < 0) then
                       call set_antibump_spec1(bump, atom, frag, wordv(1), wordv(2), scaled)
                       if (nrestraints(btable) > 0) call activate_bump(btable,pxen_from_specie(wordv(1)),pxen_from_specie(wordv(2)))
                   else
                       call set_antibump_spec1(bump, atom, frag, wordv(1), wordv(2), scaled, targ)
                       if (nrestraints(btable) > 0)  &
                       call activate_bump(btable,pxen_from_specie(wordv(1)),pxen_from_specie(wordv(2)),targ)
                   endif
               endif
           endif

         case (1)  
           ier = 2

         case (2)   ! es. frag1 frag2 3.0
           nfrag = numfragments(frag)
           call getnum(line,vet,ivet,iv)
           select case(iv)       
             case (2,3)
               if (ivet(1) > nfrag .or. ivet(2) > nfrag .or. ivet(1) < 1 .or. ivet(2) < 1) then
                   ier = 3
               else
                   if (iv == 2) then
                       call set_antibump_frag(bump,atom,frag(ivet(1)),frag(ivet(2)),scaled)
                   elseif (iv == 3) then
                       call set_antibump_frag(bump,atom,frag(ivet(1)),frag(ivet(2)),scaled,vet(3))
                   endif
               endif
             case default
               ier = 3
           end select

      end select
   else
       call set_antibump_auto(bump, btable, atom, frag, scaled)
   endif
!
   end subroutine bump_from_string

!--------------------------------------------------------------

   subroutine set_antibump_spec(bump,atom,frag,spec,scaled,targ)
!
!  Set anti-bump restraints for for species in the array spec
!
   USE atom_type_util
   USE fragmentmod
   USE General, only: xgg
   USE cgeom
   type(restraint_type), dimension(:), allocatable, intent(inout) :: bump
   type(atom_type), dimension(:), allocatable, intent(in)         :: atom
   type(fragment_type), dimension(:), allocatable, intent(in)     :: frag
   character(len=*), dimension(:), intent(in)                     :: spec
   real, intent(in)                                               :: scaled
   real, intent(in), optional                                     :: targ
   integer                                                        :: nbump, nspec, nat
   integer                                                        :: i,j
   type(restraint_type)                                           :: bump_new
   real                                                           :: tdist
   integer                                                        :: respos
   integer                                                        :: ntypespec
!
   nbump = nrestraints(bump)
!
!  allocate bump to a reasonable dimension
   nspec = 0
   ntypespec = size(spec)
   do i=1,ntypespec
      nspec = nspec + numatomspec(atom,spec(i))
   enddo
   call reallocate_restr(bump,nbump+nspec+nspec*(nspec-1)/2)
!   
   nat = numatomtype(atom)
!
   do i=1, nat
      do j=i, nat
!
!        check if species of i and j are both in the spec array
         if (is_atomic_specie(atom(i),spec) .and. is_atomic_specie(atom(j),spec)) then
             if (present(targ)) then
                 tdist = targ * scaled
             else
                 !tdist = SCALEDIST*(vdw_radius(atomic_number(atom(i))) + vdw_radius(atomic_number(atom(j))))
                 tdist = bump_distance(atomic_number(atom(i)), atomic_number(atom(j))) * scaled
             endif
             if (fragment_pos(frag,i) == fragment_pos(frag,j)) then                  ! if in the same fragment
                 bump_new = restraint_type(4,(/-i,j/),distanzaC(atom(i)%xc,atom(j)%xc,xgg),tdist,SIGMADEF) ! only equivalent position are checked
             else
                 bump_new = restraint_type(4,(/i,j/),distanzaC(atom(i)%xc,atom(j)%xc,xgg),tdist,SIGMADEF)
             endif
             respos = restraint_position(bump,bump_new)
             if (respos == 0) then  ! add anti-bump in a list
                 nbump = nbump + 1
                 bump(nbump) = bump_new
             else                   ! replace existing anti-bump
                 bump(respos) = bump_new
             endif
         endif
      enddo
   enddo
!
   call reallocate_restr(bump,nbump)
!
   end subroutine set_antibump_spec

!--------------------------------------------------------------

   subroutine set_antibump_frag(bump,atom,frag1,frag2,scaled,targ)
!
!  Set anti-bump restraint between 2 molecular fragments
!
   USE atom_type_util
   USE fragmentmod
   USE General, only: xgg
   USE cgeom
   type(restraint_type), dimension(:), allocatable, intent(inout) :: bump
   type(atom_type), dimension(:), allocatable, intent(in)         :: atom
   type(fragment_type), intent(in)                                :: frag1, frag2
   real, intent(in)                                               :: scaled
   integer                                                        :: nbump, nat
   integer                                                        :: i,j
   type(restraint_type)                                           :: bump_new
   real, intent(in), optional                                     :: targ
   real                                                           :: tdist
   integer                                                        :: respos
   logical                                                        :: equal_fragment 
   integer                                                        :: n1,n2
!
   nbump = nrestraints(bump)
!
!  allocate bump to a reasonable dimension
   call reallocate_restr(bump,nbump+frag1%nat*frag2%nat)
!   
   nat = numatomtype(atom)
   equal_fragment = any(frag1%pos(:) == frag2%pos(1))  ! are the same fragment?
!
   do i=1, frag1%nat
      do j=1, frag2%nat
         n1 = frag1%pos(i)
         n2 = frag2%pos(j)
         if (present(targ)) then
             tdist = targ * scaled
         else
             !tdist = SCALEDIST*(vdw_radius(atomic_number(atom(n1))) + vdw_radius(atomic_number(atom(n2))))
             tdist = bump_distance(atomic_number(atom(n1)), atomic_number(atom(n2))) * scaled
         endif
         if (equal_fragment) then                  ! if in the same fragment
             bump_new = restraint_type(4,(/-n1,n2/),distanzaC(atom(n1)%xc,atom(n2)%xc,xgg),tdist,SIGMADEF) ! only equivalent position are checked
         else
             bump_new = restraint_type(4,(/n1,n2/),distanzaC(atom(n1)%xc,atom(n2)%xc,xgg),tdist,SIGMADEF)
         endif
         respos = restraint_position(bump,bump_new)
         if (respos == 0) then  ! add anti-bump in a list
             nbump = nbump + 1
             bump(nbump) = bump_new
         else                   ! replace existing anti-bump
             bump(respos) = bump_new
         endif
      enddo
   enddo
!
   call reallocate_restr(bump,nbump)
!
   end subroutine set_antibump_frag

!-----------------------------------------------------------------------------------

   subroutine set_antibump_spec1(bump,atom,frag,spec1,spec2,scaled,targ)
!
!  Set all bump restraints between species spec1 spec2
!
   USE atom_type_util
   USE fragmentmod
   USE General, only: xgg
   USE cgeom
   type(restraint_type), dimension(:), allocatable, intent(inout) :: bump
   type(atom_type), dimension(:), allocatable, intent(in)         :: atom
   type(fragment_type), dimension(:), allocatable, intent(in)     :: frag
   character(len=*), intent(in)                                   :: spec1, spec2
   type(restraint_type)                                           :: bump_new
   integer                                                        :: nbump
   integer                                                        :: nspec1, nspec2
   real, intent(in)                                               :: scaled
   real, intent(in), optional                                     :: targ
   real                                                           :: tdist
   integer                                                        :: respos
   integer                                                        :: i,j
   integer                                                        :: nat
!
   nbump = nrestraints(bump)
   nspec1 = numatomspec(atom,spec1)
   if (nspec1 == 0) return
   nspec2 = numatomspec(atom,spec2)
   if (nspec2 == 0) return
   call reallocate_restr(bump,nbump+nspec1*nspec2)
!
   nat = numatomtype(atom)
   do i=1, nat
      do j=i, nat
         if ((is_atomic_specie(atom(i),spec1) .and. is_atomic_specie(atom(j),spec2)) .or.   &
             (is_atomic_specie(atom(i),spec2) .and. is_atomic_specie(atom(j),spec1))) then
             if (present(targ)) then
                 tdist = targ * scaled
             else
                 tdist = bump_distance(atomic_number(atom(i)), atomic_number(atom(j))) * scaled
             endif
             if (fragment_pos(frag,i) == fragment_pos(frag,j)) then                  ! if in the same fragment
                 bump_new = restraint_type(4,(/-i,j/),distanzaC(atom(i)%xc,atom(j)%xc,xgg),tdist,SIGMADEF) ! only equivalent position are checked
             else
                 bump_new = restraint_type(4,(/i,j/),distanzaC(atom(i)%xc,atom(j)%xc,xgg),tdist,SIGMADEF)
             endif
!
             respos = restraint_position(bump,bump_new)
             if (respos == 0) then  ! add anti-bump in a list
                 nbump = nbump + 1
                 bump(nbump) = bump_new
             else                   ! replace existing anti-bump
                 bump(respos) = bump_new
             endif
         endif
      enddo
   enddo
!
   call reallocate_restr(bump,nbump)
!
   end subroutine set_antibump_spec1
   
!--------------------------------------------------------------

   subroutine print_antibump(kpr,bump,atom)
!
!  Print list of anti-bumping restraints
!
   USE atom_type_util
   USE connect_mod
   integer, intent(in)                                    :: kpr
   type(restraint_type), dimension(:), allocatable, intent(in) :: bump
   type(atom_type), dimension(:), allocatable, intent(in) :: atom
   if (nrestraints(bump) > 0) then
       call print_restraints(kpr,bump,ABUMP,atom)
   endif
   end subroutine print_antibump

!--------------------------------------------------------------

   real function antibump_function(bump,atom,doc,kpr) result(bfunc)
!
!  Compute cost function for anti-bump restraints
!
   USE atom_type_util
   USE fragmentmod
   USE Symmetry, only: nsymtot
   USE general, only: xgg
   USE connect_mod
   USE cgeom
   type(restraint_type), dimension(:), allocatable, intent(inout) :: bump
   type(atom_type), dimension(:), allocatable, intent(in)    :: atom
   logical, intent(in)                                            :: doc
   logical, intent(in) :: kpr
   integer :: nbump
   integer :: n1, n2
   real :: dd
   type(atom_type) :: at
   integer :: i, k
   real, dimension(3) :: ktra
   logical :: same_frag
   real :: dist
   real, parameter :: EPS = epsilon(1.0)
!
   bfunc = 0
   nbump = nrestraints(bump)
   do i=1,nbump
      n1 = bump(i)%na(1)
      n2 = bump(i)%na(2)
      if (n1 < 0) then
          n1 = abs(n1)
          same_frag = .true.
      else
          same_frag = .false.
      endif
      !if (doc .and. (abs(atom(n1)%ocry - 1.0) > EPS .or. abs(atom(n2)%ocry - 1.0) > EPS)) then
      if (doc .and. (atom(n1)%nz == atom(n2)%nz)) then
           !write(0,*)'ocry=',trim(atom(n1)%lab)//'-'//trim(atom(n2)%lab),atom(n1)%ocry,atom(n2)%ocry
           cycle
      endif
      dist = distanzaC(atom(n2)%xc,atom(n1)%xc,xgg)
      bump(i)%val = dist   !! FIXME - meglio mettere la distanza minima
      do k=1,nsymtot
         at = atom_symm(k, atom(n1))
         call xdisteqs(atom(n2)%xc,at%xc,xgg,dd,ktra)    ! also for start = 2 and k = 1 to find equivalent for traslation
         if (dd < bump(i)%targ) then
             bump(i)%val = dd
             if (same_frag .and. k == 1) then           ! n1 and n2 are in the same fragment when k==1
                 if (atom(n2) + ktra == atom(n1)) cycle  ! if true n1 is not an equivalent for translation
                       !if (same_frag .and. abs(dd -dist) < 0.05 .and. k==1) then
                       !    write(70,*)'op=',trim(atom(n2)%lab),trim(atom(n1)%lab)
                       !    write(70,*)'at1=',atom(n1)%xc
                       !    write(70,*)'at2=',atom(n2)%xc
                       !    write(70,*)'at=',at%xc
                       !    write(70,*)'at=',atom(n2)%xc + ktra
                       !    write(70,*)'ktra=',atom(n1)%xc - atom(n2)%xc
                       !    write(70,*)'op=',k,ktra
                       !    write(70,*)'TRA=',all(abs(ktra - atom(n1)%xc + atom(n2)%xc) < 0.001)
                       !endif
             endif
             !if (.not. (same_frag .and. abs(dd - dist) < 0.05)) then
                 bfunc = bfunc + ((1/bump(i)%sigma)*(dd - bump(i)%targ))**4 
                 if(kpr) then
!corr                    write(0,*)'activate bump:',atom(n1)%lab,atom(n2)%lab,dd,(1/bump(i)%sigma)*(dd - bump(i)%targ)**2
                    write(6,*)'activate bump:',atom(n1)%lab,atom(n2)%lab,dd,(1/bump(i)%sigma)*(dd - bump(i)%targ)**2
                 endif
             !endif
         endif
      enddo
   enddo
   if (kpr) then
!corr       write(0,*)'fbump=',bfunc
       write(6,*)'fbump=',bfunc
   endif
!
   end function antibump_function

END MODULE anti_bump

   subroutine test_bump()
   USE variables
   USE fragmentmod
   USE connect_mod
   USE anti_bump
   type(fragment_type), dimension(:), allocatable :: frag
   type(restraint_type), dimension(:), allocatable     :: bump
   type(bump_settings) :: bset
   integer :: nfrag
   real :: fbump
          write(0,*)'start test_bump',size(atm),size(lconn)
   call get_fragments(atm,lconn,nfrag,frag)
   !call set_antibump_auto(bump, atm, frag)
        call set_antibump_spec1(bump, atm, frag, 'C', 'C',bset%scaledist)
        call set_antibump_spec1(bump, atm, frag, 'C', 'H',bset%scaledist)
        call set_antibump_spec1(bump, atm, frag, 'O', 'O',bset%scaledist)
        !call set_antibump_spec1(bump, atm, frag, 'C', 'O', 2.4)
        !call set_antibump_spec1(bump, atm, frag, 'O', 'O', 2.3)
        !call set_antibump_spec1(bump, atm, frag, 'V', 'C')
        !call set_antibump_spec1(bump, atm, frag, 'V', 'C')
        !call set_antibump_spec1(bump, atm, frag, 'O', 'O',2.0)
   !call set_antibump_spec1(bump, atm, frag, 'C', 'H')
   !call set_antibump_spec1(bump, atm, frag, 'H', 'H')
   !call set_antibump_spec1(bump, atm, frag, 'V', 'C')
   !call set_antibump_spec1(bump, atm, frag, 'V', 'V')
   !call set_antibump_spec1(bump, atm, frag, 'O', 'H')  !-- prosssimo test
   fbump = antibump_function(bump, atm, .false.,.false.)
   call print_antibump(0,bump,atm)
   fbump = antibump_function(bump, atm, .false., .true.)
   end subroutine test_bump

